﻿using System;
using System.IO;
using System.Runtime.CompilerServices;
using System.Text;
using System.Xml;
using System.Linq;
using System.Collections.Generic;
using DataContracts;
using System.Text.RegularExpressions;
using Ato.EN.IntegrationServices.CodeGenerationCGTS;
using Ato.EN.IntegrationServices.CodeGenerationDIDS;
using Ato.EN.IntegrationServices.CodeGenerationNRFI;
using Ato.EN.IntegrationServices.CodeGenerationPSS;
using Ato.EN.IntegrationServices.CodeGenerationRS;
using Ato.EN.IntegrationServices.CodeGenerationWRE;

namespace Ato.EN.IntegrationServices.CodeGenerationIITR
{
    public partial class IITR2018CrossformValidator
    {
        /// <summary>
        /// The max parameter name length is restricted by the SBR1 and SBR2 schemas - the lowest common denominator of SBR 1 is used as the default
        /// </summary>
        private int _maxParameterNameLength;

        /// <summary>
        /// The max parameter value length is restricted by the SBR1 and SBR2 schemas
        /// </summary>
        private int _maxParameterValueLength;

        /// <summary>
        /// The SBR1 and ebms3 schemas do not allow parameter names or values to be the empty string
        /// </summary>
        private string _emptyParameterValue;

        /// <summary>
        /// Initializes a new instance of the <see cref="IITR2018CrossformValidator" /> class.
        /// </summary>
        /// <param name="maxParameterNameLength">Maximum length of the parameter name.</param>
        /// <param name="maxParameterValueLength">Maximum length of the parameter value.</param>
        /// <param name="emptyParameterValue">This value will be used in place of any parameter values that result in a null or empty value.</param>
        public IITR2018CrossformValidator(int maxParameterNameLength = 20, int maxParameterValueLength = 4096, string emptyParameterValue = "EMPTY")
        {
            _maxParameterNameLength = maxParameterNameLength;
            _maxParameterValueLength = maxParameterValueLength;
            _emptyParameterValue = emptyParameterValue;
        }

        #region Functions
        private static IEnumerable<string> Union(IEnumerable<string> list1, IEnumerable<string> list2)
        {
            IEnumerable<string> response;

            if (list1 == null && list2 == null)
            {
                response = null;
            }
            else if (list1 == null)
            {
                response = list2.Distinct();
            }
            else if (list2 == null)
            {
                response = list1.Distinct();
            }
            else
            {
                response = list1.Union(list2);
            }

            return response;
        }

        private static bool IsMatch(int? field, string expression, RegexOptions options = RegexOptions.None)
        {
            if (field == null)
                return false;
            else
                return Regex.IsMatch(Convert.ToString(field.Value), expression, options);
        }

        private static bool IsMatch(string field, string expression, RegexOptions options = RegexOptions.None)
        {
            if (field == null)
                return false;
            else
                return Regex.IsMatch(field, expression, options);
        }

        // This is just context and tuple counts where they are integer values - easier that changing the parsing logic to just return the value
        private static int Count(int count)
        {
            return count;
        }


        private static int Count<T>(IEnumerable<T> values)
        {
            return values == null ? 0 : values.Where(f => f != null).Count();
        }


        private static int Count<T>(ICollection<T> values)
        {
            return values == null ? 0 : values.Where(f => f != null).Count();
        }


        private static bool exists(bool value)
        {
            return value;
        }

        private static string GetValueOrEmpty(bool? val)
        {
            return (val.HasValue) ? val.ToString() : string.Empty;
        }

        private static string GetValueOrEmpty(DateTime? val)
        {
            return (val.HasValue) ? val.ToString() : string.Empty;
        }

        /// <summary>
        /// Get string value between [first] a and [last] b.
        /// </summary>
        public static string Between(string value, string a, string b)
        {
            int posA = value.IndexOf(a);
            int posB = value.LastIndexOf(b);
            if (posA == -1)
            {
                return "";
            }
            if (posB == -1)
            {
                return "";
            }
            int adjustedPosA = posA + a.Length;
            if (adjustedPosA >= posB)
            {
                return "";
            }
            return value.Substring(adjustedPosA, posB - adjustedPosA);
        }

        /// <summary>
        /// Get string value after [first] a.
        /// </summary>
        public static string Before(string value, string a)
        {
            int posA = value.IndexOf(a);
            if (posA == -1)
            {
                return "";
            }
            return value.Substring(0, posA);
        }

        /// <summary>
        /// Get string value after [last] a.
        /// </summary>
        public static string After(string value, string a)
        {
            int posA = value.LastIndexOf(a);
            if (posA == -1)
            {
                return "";
            }
            int adjustedPosA = posA + a.Length;
            if (adjustedPosA >= value.Length)
            {
                return "";
            }
            return value.Substring(adjustedPosA);
        }

        private static int Length(object field)
        {
            if (field == null)
                return 0;
            else
                return field.ToString().Trim().Length;
        }

        private static bool NotSameValues(IEnumerable<object> nodes)
        {
            if (nodes == null)
                return false;

            object[] nodesArray = nodes.Cast<object>().ToArray();
            return NotSameValues(nodesArray);
        }

        private static bool NotSameValues(params object[] nodes)
        {
            if (nodes == null)
                return false;

            return ((from x in nodes select x).Distinct().Count() == nodes.Count());
        }

        private static bool HasDuplicateValues(IEnumerable<object> nodes)
        {
            if (nodes == null)
                return false;

            object[] nodesArray = nodes.Cast<object>().ToArray();
            return HasDuplicateValues(nodesArray);
        }

        private static bool HasDuplicateValues(params object[] nodes)
        {
            if (nodes == null)
                return false;

            return !((from x in nodes select x).Distinct().Count() == nodes.Count());
        }

        private int DuplicateValueIndex(IEnumerable<object> values)
        {
            int response = 0;
            var hashset = new HashSet<object>();
            foreach (var value in values)
            {
                if (!hashset.Add(value))
                {
                    return response;
                }
                response++;
            }
            return response;
        }

        private int DuplicateValueIndex<T>(IEnumerable<T?> values) where T : struct
        {
            int response = 0;
            var hashset = new HashSet<T?>();
            foreach (var value in values)
            {
                if (!hashset.Add(value))
                {
                    return response;
                }
                response++;
            }
            return response;
        }

        private static bool IsDate(object value)
        {
            DateTime dateValue;
            return (value != null && DateTime.TryParse(value.ToString(), out dateValue));
        }

        public DateTime AsDate(string dateAsString)
        {
            DateTime response = DateTime.MinValue;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date;
            }

            return response;
        }
        //The logic in After function expects "---" for day and "--" for month. 
        //Since hyphen was missing the date always returned null
        public DateTime? ConvertToDate(int day, int month, int year)
        {
            return ConvertToDate(day == 0 ? null : "---" + day.ToString(), month == 0 ? null : "--" + month.ToString(), year == 0 ? null : year.ToString());
        }

        public DateTime? ConvertToDate(string day, string month, string year)
        {
            DateTime? response;
            DateTime result;

            if (year == null || month == null || day == null)
            {
                return null;
            }
            string dateAsString = year + "-" + After(month, "--") + "-" + After(day, "---");
            if (DateTime.TryParse(dateAsString, out result))
            {
                response = result;
            }
            else
            {
                response = null;
            }
            return response;
        }


        public DateTime? ConvertToDate(string day, string month, int year)
        {
            DateTime? response;
            DateTime result;

            if (year == 0 || month == null || day == null)
            {
                return null;
            }
            string dateAsString = year.ToString() + "-" + After(month, "--") + "-" + After(day, "---");
            if (DateTime.TryParse(dateAsString, out result))
            {
                response = result;
            }
            else
            {
                response = null;
            }
            return response;
        }


        private static int Day(string dateAsString)
        {
            int response = 0;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date.Day;
            }

            return response;
        }


        private static int Day(DateTime? date)
        {
            if (date == null)
                return 0;
            else
                return date.Value.Day;

        }


        private static string Month(string dateAsString)
        {
            string response = null;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date.ToString("MMMM");
            }
            else
            {
                return "NotAMonth";
            }

            return response;
        }

        private static string Month(DateTime? date)
        {
            if (date == null)
                return "NotAMonth";
            else
                return date.Value.ToString("MMMM");
        }


        private static int MonthAsInt(string dateAsString)
        {
            int response = 0;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date.Month;
            }

            return response;
        }

        private static int MonthAsInt(DateTime? date)
        {
            if (date == null)
            {
                return 0;
            }
            return date.Value.Month;
        }


        private static int Year(string dateAsString)
        {
            int response = 0;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
            {
                response = date.Year;
            }

            return response;
        }

        private static int Year(DateTime? date)
        {
            if (date == null)
                return 0;
            else
                return date.Value.Year;
        }


        private static int CurrentFinancialYear()
        {
            return DateToFinancialYear(DateTime.Now, 7);
        }

        private static int FinancialYear(string dateAsString)
        {
            return DateToFinancialYear(dateAsString, 7);
        }

        private static int FinancialYear(DateTime? date)
        {
            return DateToFinancialYear(date, 7);
        }

        private static int DateToFinancialYear(string dateAsString, int startingMonth)
        {
            int response = 0;
            DateTime date;
            if (DateTime.TryParse(dateAsString, out date))
            {
                response = DateToFinancialYear(date, startingMonth);
            }

            return response;
        }
        private static int DateToFinancialYear(DateTime? date, int startingMonth)
        {
            int response;
            if (date == null)
            {
                response = 0;
            }
            else
            {
                int year = date.Value.Year;
                int month = date.Value.Month;

                if (startingMonth > month)
                    response = year;
                else
                    response = year + 1;
            }
            return response;
        }


        private static int FBTYear(string dateAsString)
        {
            int response = 0;
            DateTime date;

            if (DateTime.TryParse(dateAsString, out date))
                response = FBTYear(date);
            return response;
        }

        private static int FBTYear(DateTime? date)
        {
            if (date == null)
            {
                return 0;
            }
            else
            {
                if (date.Value.Month > 3)
                    return date.Value.Year + 1;
                else
                    return date.Value.Year;
            }
        }


        private static bool IsNumeric(object value)
        {
            decimal numbervalue;
            return (value != null && decimal.TryParse(value.ToString(), out numbervalue));
        }

        private static bool NotMonetary(decimal? field, string sign, int digits, int decimals)
        {
            if (field == null)
            {
                return false;
            }
            else
            {
                string signExpression;
                string decimalExpression;
                int digitsToUse = digits - decimals;

                if (sign == "U")
                    signExpression = "^";
                else
                    signExpression = "^-?";

                if (decimals > 0)
                    decimalExpression = @"(\.\d{1," + decimals + "})?$";
                else
                    decimalExpression = @"$";

                return !(Regex.IsMatch(field.Value.ToString("0.#########################"), signExpression + @"\d{1," + digitsToUse + "}" + decimalExpression));
            }
        }

        private static bool NotNumeric(decimal? field, string sign, int digits, int decimals)
        {
            if (field == null)
            {
                return false;
            }
            else
            {
                string signExpression;
                string decimalExpression;
                int digitsToUse = digits - decimals;

                if (sign == "U")
                    signExpression = "^";
                else
                    signExpression = "^-?";

                if (decimals > 0)
                    decimalExpression = @"(\.\d{1," + decimals + "})?$";
                else
                    decimalExpression = @"$";

                return !(Regex.IsMatch(field.Value.ToString("0.#########################"), signExpression + @"\d{1," + digitsToUse + "}" + decimalExpression));
            }
        }

        private static bool NotNumeric(int? field, string sign, int digits, int decimals = 0)
        {
            if (field == null)
            {
                return false;
            }
            else
            {
                string signExpression;

                if (sign == "U")
                    signExpression = "^";
                else
                    signExpression = "^-?";

                return !(Regex.IsMatch(field.Value.ToString(), signExpression + @"\d{1," + digits + "}$"));
            }
        }

        private static bool NotNumeric(long? field, string sign, int digits, int decimals = 0)
        {
            if (field == null)
            {
                return false;
            }
            else
            {
                string signExpression;

                if (sign == "U")
                    signExpression = "^";
                else
                    signExpression = "^-?";

                return !(Regex.IsMatch(field.Value.ToString(), signExpression + @"\d{1," + digits + "}$"));
            }
        }


        private static bool OutsideRange(decimal field, decimal expression, int range)
        {
            bool response;

            response = (field < (expression - range)) || (field > (expression + range));

            return response;
        }


        private static bool FailsUSIAlgorithm(string usi, string abn)
        {
            bool response;
            if (usi == null || abn == null)
            {
                response = false;
            }
            else
            {
                usi = usi.Trim();
                abn = abn.Trim();
                if (usi.Length < 13 || abn.Length < 11)
                {
                    response = false;
                }
                else
                {
                    int numeric;
                    if (usi.Substring(0, 11) == abn && int.TryParse(usi.Substring(11, 2), out numeric))
                        response = false;
                    else if (Regex.IsMatch(usi, @"^[a-zA-Z]{3}\d{4}[a-zA-Z]{2}"))
                        response = false;
                    else
                        response = true;
                }
            }
            return response;
        }


        private static bool FailsTANAlgorithm(string tan)
        {
            bool response;
            decimal decimalTan;

            if (tan == null)
                return false;

            tan = tan.Trim();

            if (!decimal.TryParse(tan, out decimalTan))
                return true;

            if (tan.Length != 8)
                return true;

            decimal tanSum =
                7 * int.Parse(tan.Substring(0, 1)) +
                9 * int.Parse(tan.Substring(1, 1)) +
                8 * int.Parse(tan.Substring(2, 1)) +
                4 * int.Parse(tan.Substring(3, 1)) +
                6 * int.Parse(tan.Substring(4, 1)) +
                3 * int.Parse(tan.Substring(5, 1)) +
                5 * int.Parse(tan.Substring(6, 1)) +
                1 * int.Parse(tan.Substring(7, 1));

            if ((tanSum % 11) == 0)
                response = false;
            else
                response = true;

            return response;
        }


        private static bool FailsABNAlgorithm(string abn)
        {
            bool response;
            decimal decimalAbn;

            if (abn == null)
                return false;

            abn = abn.Trim();

            if (!decimal.TryParse(abn, out decimalAbn))
                return true;

            if (abn.Length != 11)
                return true;

            decimal abnSum =
                10 * (int.Parse(abn.Substring(0, 1)) - 1) +
                1 * int.Parse(abn.Substring(1, 1)) +
                3 * int.Parse(abn.Substring(2, 1)) +
                5 * int.Parse(abn.Substring(3, 1)) +
                7 * int.Parse(abn.Substring(4, 1)) +
                9 * int.Parse(abn.Substring(5, 1)) +
                11 * int.Parse(abn.Substring(6, 1)) +
                13 * int.Parse(abn.Substring(7, 1)) +
                15 * int.Parse(abn.Substring(8, 1)) +
                17 * int.Parse(abn.Substring(9, 1)) +
                19 * int.Parse(abn.Substring(10, 1));

            if ((abnSum % 89) == 0)
                response = false;
            else
                response = true;

            return response;
        }
        private static bool FailsACNAlgorithm(string acn)
        {
            bool response;
            decimal decimalAbn;
            if (acn == null)
                return false;

            acn = acn.Trim();

            if (!decimal.TryParse(acn, out decimalAbn))
                return true;

            if (acn.Length != 9)
                return true;

            decimal abnSum =
                8 * int.Parse(acn.Substring(0, 1)) +
                7 * int.Parse(acn.Substring(1, 1)) +
                6 * int.Parse(acn.Substring(2, 1)) +
                5 * int.Parse(acn.Substring(3, 1)) +
                4 * int.Parse(acn.Substring(4, 1)) +
                3 * int.Parse(acn.Substring(5, 1)) +
                2 * int.Parse(acn.Substring(6, 1)) +
                1 * int.Parse(acn.Substring(7, 1));

            decimal checkDigit = int.Parse(acn.Substring(8, 1));
            decimal acnRemainder = abnSum % 10;

            if (((10 - acnRemainder) % 10) == checkDigit)
                response = false;
            else
                response = true;

            return response;
        }


        private static bool FailsTFNAlgorithm(string tfn)
        {
            bool response;
            decimal decimalTfn;

            if (tfn == null)
                return false;

            tfn = tfn.Trim();
            tfn = Regex.Replace(tfn, "^0+", "");

            if (!decimal.TryParse(tfn, out decimalTfn))
                return true;

            if (tfn.Length < 8)
                return true;


            decimal tfn1To7Sum =
                1 * int.Parse(tfn.Substring(0, 1)) +
                4 * int.Parse(tfn.Substring(1, 1)) +
                3 * int.Parse(tfn.Substring(2, 1)) +
                7 * int.Parse(tfn.Substring(3, 1)) +
                5 * int.Parse(tfn.Substring(4, 1)) +
                8 * int.Parse(tfn.Substring(5, 1)) +
                6 * int.Parse(tfn.Substring(6, 1));

            decimal tfn8 = 9 * int.Parse(tfn.Substring(7, 1));

            if (tfn.Length == 8)
            {
                decimal tFNLg8WSum9 = 10 * int.Parse(tfn.Substring(7, 1));
                decimal tFNLg8WSum = tfn1To7Sum + tFNLg8WSum9;

                if ((tFNLg8WSum % 11) == 0)
                    response = false;
                else
                    response = true;
            }
            else if (tfn.Length == 9)
            {
                decimal tfn9 = 10 * int.Parse(tfn.Substring(8, 1));
                decimal tFNLg9WSum = tfn1To7Sum + tfn8 + tfn9;

                if ((tFNLg9WSum % 11) == 0)
                    response = false;
                else
                    response = true;
            }
            else
            {
                response = true;
            }

            return response;
        }


        private static decimal ConditionalValue(bool expression, decimal? trueVal, decimal? falseVal)
        {
            return expression ? trueVal.GetValueOrDefault() : falseVal.GetValueOrDefault();
        }

        private static decimal AsNumeric(string value)
        {
            decimal numberValue;
            decimal.TryParse(value, out numberValue);
            return numberValue;
        }

        private static bool RegexMatch(int? field, string expression, string flags = "")
        {
            return IsMatch(field, expression, GetRegexOption(flags));
        }

        private static bool RegexMatch(string field, string expression, string flags = "")
        {
            return IsMatch(field, expression, GetRegexOption(flags));
        }

        private static RegexOptions GetRegexOption(string flags)
        {
            RegexOptions options = RegexOptions.None;

            char[] characters = flags.ToCharArray();

            foreach (char character in characters)
            {
                switch (character)
                {
                    case 'i':
                        options = options | RegexOptions.IgnoreCase;
                        break;
                    case 'm':
                        options = options | RegexOptions.Multiline;
                        break;
                    case 's':
                        options = options | RegexOptions.Singleline;
                        break;
                    case 'n':
                        options = options | RegexOptions.ExplicitCapture;
                        break;
                    case 'x':
                        options = options | RegexOptions.IgnorePatternWhitespace;
                        break;
                }
            }

            return options;
        }

        /// <summary>
        /// Returns an occurrence index as [occurrenceIndex] of occurrenceIndex > 0, otherwise the empty string
        /// </summary>
        /// <param name="occurrenceIndex">Index of the occurrence.</param>
        /// <returns>Occurrence in XPath [#] format</returns>
        public string OccurrenceIndex(int occurrenceIndex)
        {
            return occurrenceIndex > 0 ? "[" + occurrenceIndex + "]" : "";
        }

        #endregion // Functions

        public IITR2018Crossform ConsumedReport { get; private set; }

        private static Dictionary<string, ProcessMessageDocument> _processMessageDocuments = new Dictionary<string, ProcessMessageDocument>();

        [MethodImpl(MethodImplOptions.AggressiveInlining)]
        public List<ProcessMessageDocument> ValidateReport(IITR2018Crossform report)
        {
            List<ProcessMessageDocument> response = new List<ProcessMessageDocument>();
            ProcessMessageDocument processMessage;
            bool assertion;

            this.ConsumedReport = report;

			#region VR.ATO.IITR.310090

            /*  VR.ATO.IITR.310090
            Where the short form Income Tax Return Type is being used, no attached schedules are allowed, and no entries for Employment termination payment or Superannuation lump sum payment details are permitted.
    
            Legacy Rule Format:
            IF [iitr13] <> NULL AND (COUNT(SCHEDULE = "RS") > 0 OR COUNT(SCHEDULE = "CGTS") = 1 OR COUNT(SCHEDULE = "NRFI") = 1 OR COUNT(SCHEDULE = "PSS") = 1 OR COUNT(SCHEDULE = "DIDS") > 0 OR COUNT(SCHEDULE = "WRE") = 1 OR COUNT(RP.{ETPSeqNum}) > 0 OR COUNT(RP.{SLSPSeqNum}) > 0)
			   RETURN VALIDATION MESSAGE
			ENDIF

            Data Elements:
    
            IITR:RP:^IITR13 = tns:IncomeTax.ReturnType.Code
            */
            assertion = (report.iitrParentDocument.IITR13 != null && (Count(report.rsChildDocuments) > 0 || Count(report.cgtsChildDocuments) == 1 || Count(report.nrfiChildDocuments) == 1 || Count(report.pssChildDocuments) == 1 || Count(report.didsChildDocuments) > 0 || Count(report.wreChildDocuments) == 1 || Count(report.iitrParentDocument.RPETPSeqNumContextCollection) > 0 || Count(report.iitrParentDocument.RPSLSPSeqNumContextCollection) > 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.310090",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Attached schedules or ETP/SLSP details are invalid",
                    LongDescription = @"When the 'Income Tax Return Type' is provided, implying that the return is to be treated as a short Individual return, no attached schedules are allowed, and no entries for Employment termination payment or Superannuation lump sum payment details are permitted",
                    Location = "/xbrli:xbrl/tns:IncomeTax.ReturnType.Code",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310090" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR13", Value = report.iitrParentDocument.IITR13 });

                response.Add(processMessage);
            }

            #endregion // VR.ATO.IITR.310090

            // ####################################################################################################
            //                                         CGTS CROSSFORM RULES
            // ####################################################################################################

            #region VR.ATO.IITR.310066

            /* VR.ATO.IITR.310066
            When net capital gain is in excess of $10,000 a Capital Gains Tax schedule must be attached.

            Legacy Rule Format:
            IF ([iitr311] > 10000) AND (COUNT(SCHEDULE = "CGTS") = 0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Data Elements:

            IITR:RP:^IITR311 = tns:Capital.Gains.Net.Amount
            */
            assertion = (report.iitrParentDocument.IITR311.GetValueOrDefault() > 10000 && (report.cgtsChildDocuments == null || Count(report.cgtsChildDocuments) == 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.310066",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Capital Gains Tax schedule required",
                    LongDescription = @"When Net capital gain is in excess of $10,000, a Capital Gains Tax schedule must be attached",
                    Location = "/xbrli:xbrl/tns:Capital.Gains.Net.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310066" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR311", Value = report.iitrParentDocument.IITR311.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }

            #endregion // VR.ATO.IITR.310066

            #region VR.ATO.IITR.310067

            /* VR.ATO.IITR.310067
            When current year capital gains is in excess of $10,000 a Capital Gains Tax schedule must be attached.

            Legacy Rule Format:
            IF ([iitr312] > 10000) AND (COUNT(SCHEDULE = "CGTS") = 0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Data Elements:
            IITR:RP:^IITR312 = tns:Capital.Gains.Total.Amount
            */
            assertion = (report.iitrParentDocument.IITR312.GetValueOrDefault() > 10000 && (report.cgtsChildDocuments == null || Count(report.cgtsChildDocuments) == 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.310067",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Capital Gains Tax schedule required",
                    LongDescription = @"When Total current year capital gains is in excess of $10,000, a Capital Gains Tax schedule must be attached",
                    Location = "/xbrli:xbrl/tns:Capital.Gains.Total.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310067" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR312", Value = report.iitrParentDocument.IITR312.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }

            #endregion // VR.ATO.IITR.310067

            #region VR.ATO.IITR.310069

            /* VR.ATO.IITR.310069
            The number of Capital Gains Tax schedules must not exceed 1.

            Legacy Rule Format:
            IF COUNT(SCHEDULE = "CGTS") > 1
                RETURN VALIDATION MESSAGE
            ENDIF
            */
            assertion = (report.cgtsChildDocuments != null && Count(report.cgtsChildDocuments) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.310069",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"The number of Capital Gains Tax schedules must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310069" } },
                };

                response.Add(processMessage);
            }

            #endregion // VR.ATO.IITR.310069

            #region CGTS Child Present

            if (report.cgtsChildDocuments != null && Count(report.cgtsChildDocuments) == 1)
            {
                CGTS2018 cgtsChildDocument = report.cgtsChildDocuments[0];

                #region VR.ATO.GEN.402009

                /* VR.ATO.GEN.402009
                Your supplied TFN does not match the TFN supplied on the form it was submitted with

                Legacy Rule Format:
                IF (RP:entity.identifier.TFN <> PARENT RETURN:RP:entity.identifier.TFN)
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPIdentifierTFN != report.iitrParentDocument.RPIdentifierTFN);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.402009",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Your supplied TFN does not match the TFN supplied on the form it was submitted with",
                        LongDescription = @"Your supplied TFN does not match the TFN supplied on the parent form it was submitted with",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.402009" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.402009
                
                #region VR.ATO.GEN.438000

                /* VR.ATO.GEN.438000
                Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPStartDate != report.iitrParentDocument.RPStartDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438000",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period start date is incorrect.",
                        LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438000
                
                #region VR.ATO.GEN.438000

                /* VR.ATO.GEN.438000
                Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPAppliedExists == true && cgtsChildDocument.RPAppliedStartDate != report.iitrParentDocument.RPStartDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438000",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period start date is incorrect.",
                        LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPAppliedId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438000

                #region VR.ATO.GEN.438000

                /* VR.ATO.GEN.438000
                Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPAppliedY0Exists == true && cgtsChildDocument.RPAppliedY0StartDate != report.iitrParentDocument.RPStartDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438000",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period start date is incorrect.",
                        LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPAppliedY0Id + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438000

                #region VR.ATO.GEN.438000

                /* VR.ATO.GEN.438000
                Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPAppliedY0MinusExists == true && cgtsChildDocument.RPAppliedY0MinusStartDate != report.iitrParentDocument.RPStartDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438000",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period start date is incorrect.",
                        LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPAppliedY0MinusId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438000

                #region VR.ATO.GEN.438000

                /* VR.ATO.GEN.438000
                Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPListedCompanySharesExists == true && cgtsChildDocument.RPListedCompanySharesStartDate != report.iitrParentDocument.RPStartDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438000",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period start date is incorrect.",
                        LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPListedCompanySharesId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438000

                #region VR.ATO.GEN.438000

                /* VR.ATO.GEN.438000
                Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPOtherSharesExists == true && cgtsChildDocument.RPOtherSharesStartDate != report.iitrParentDocument.RPStartDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438000",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period start date is incorrect.",
                        LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPOtherSharesId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438000

                #region VR.ATO.GEN.438000

                /* VR.ATO.GEN.438000
                Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPListedUnitTrustsExists == true && cgtsChildDocument.RPListedUnitTrustsStartDate != report.iitrParentDocument.RPStartDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438000",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period start date is incorrect.",
                        LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPListedUnitTrustsId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438000

                #region VR.ATO.GEN.438000

                /* VR.ATO.GEN.438000
                Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPOtherUnitsExists == true && cgtsChildDocument.RPOtherUnitsStartDate != report.iitrParentDocument.RPStartDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438000",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period start date is incorrect.",
                        LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPOtherUnitsId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438000

                #region VR.ATO.GEN.438000

                /* VR.ATO.GEN.438000
                Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPAustralianRealPropertyExists == true && cgtsChildDocument.RPAustralianRealPropertyStartDate != report.iitrParentDocument.RPStartDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438000",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period start date is incorrect.",
                        LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPAustralianRealPropertyId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438000

                #region VR.ATO.GEN.438000

                /* VR.ATO.GEN.438000
                Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPOverseasRealPropertyExists == true && cgtsChildDocument.RPOverseasRealPropertyStartDate != report.iitrParentDocument.RPStartDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438000",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period start date is incorrect.",
                        LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPOverseasRealPropertyId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438000

                #region VR.ATO.GEN.438000

                /* VR.ATO.GEN.438000
                Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPCapitalGainsTrustDistributionsExists == true && cgtsChildDocument.RPCapitalGainsTrustDistributionsStartDate != report.iitrParentDocument.RPStartDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438000",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period start date is incorrect.",
                        LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPCapitalGainsTrustDistributionsId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438000

                #region VR.ATO.GEN.438000

                /* VR.ATO.GEN.438000
                Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPCollectablesExists == true && cgtsChildDocument.RPCollectablesStartDate != report.iitrParentDocument.RPStartDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438000",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period start date is incorrect.",
                        LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPCollectablesId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438000

                #region VR.ATO.GEN.438000

                /* VR.ATO.GEN.438000
                Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPOtherCapitalAssetsExists == true && cgtsChildDocument.RPOtherCapitalAssetsStartDate != report.iitrParentDocument.RPStartDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438000",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period start date is incorrect.",
                        LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPOtherCapitalAssetsId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438000
                
                #region VR.ATO.GEN.438000

                /* VR.ATO.GEN.438000
                Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPTransfersToOtherClassesExists == true && cgtsChildDocument.RPTransfersToOtherClassesStartDate != report.iitrParentDocument.RPStartDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438000",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period start date is incorrect.",
                        LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPTransfersToOtherClassesId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438000

                #region VR.ATO.GEN.438001

                /* VR.ATO.GEN.438001
                Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPEndDate != report.iitrParentDocument.RPEndDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438001",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period end date is incorrect.",
                        LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438001

                #region VR.ATO.GEN.438001

                /* VR.ATO.GEN.438001
                Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPAppliedExists == true && cgtsChildDocument.RPAppliedEndDate != report.iitrParentDocument.RPEndDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438001",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period end date is incorrect.",
                        LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPAppliedId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438001

                #region VR.ATO.GEN.438001

                /* VR.ATO.GEN.438001
                Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPAppliedY0Exists == true && cgtsChildDocument.RPAppliedY0EndDate != report.iitrParentDocument.RPEndDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438001",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period end date is incorrect.",
                        LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPAppliedY0Id + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438001

                #region VR.ATO.GEN.438001

                /* VR.ATO.GEN.438001
                Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPAppliedY0MinusExists == true && cgtsChildDocument.RPAppliedY0MinusEndDate != report.iitrParentDocument.RPEndDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438001",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period end date is incorrect.",
                        LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPAppliedY0MinusId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438001

                #region VR.ATO.GEN.438001

                /* VR.ATO.GEN.438001
                Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPListedCompanySharesExists == true && cgtsChildDocument.RPListedCompanySharesEndDate != report.iitrParentDocument.RPEndDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438001",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period end date is incorrect.",
                        LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPListedCompanySharesId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438001

                #region VR.ATO.GEN.438001

                /* VR.ATO.GEN.438001
                Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPOtherSharesExists == true && cgtsChildDocument.RPOtherSharesEndDate != report.iitrParentDocument.RPEndDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438001",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period end date is incorrect.",
                        LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPOtherSharesId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438001

                #region VR.ATO.GEN.438001

                /* VR.ATO.GEN.438001
                Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPListedUnitTrustsExists == true && cgtsChildDocument.RPListedUnitTrustsEndDate != report.iitrParentDocument.RPEndDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438001",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period end date is incorrect.",
                        LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPListedUnitTrustsId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438001

                #region VR.ATO.GEN.438001

                /* VR.ATO.GEN.438001
                Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPOtherUnitsExists == true && cgtsChildDocument.RPOtherUnitsEndDate != report.iitrParentDocument.RPEndDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438001",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period end date is incorrect.",
                        LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPOtherUnitsId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438001

                #region VR.ATO.GEN.438001

                /* VR.ATO.GEN.438001
                Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPAustralianRealPropertyExists == true && cgtsChildDocument.RPAustralianRealPropertyEndDate != report.iitrParentDocument.RPEndDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438001",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period end date is incorrect.",
                        LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPAustralianRealPropertyId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438001

                #region VR.ATO.GEN.438001

                /* VR.ATO.GEN.438001
                Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPOverseasRealPropertyExists == true && cgtsChildDocument.RPOverseasRealPropertyEndDate != report.iitrParentDocument.RPEndDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438001",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period end date is incorrect.",
                        LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPOverseasRealPropertyId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438001

                #region VR.ATO.GEN.438001

                /* VR.ATO.GEN.438001
                Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPCapitalGainsTrustDistributionsExists == true && cgtsChildDocument.RPCapitalGainsTrustDistributionsEndDate != report.iitrParentDocument.RPEndDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438001",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period end date is incorrect.",
                        LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPCapitalGainsTrustDistributionsId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438001

                #region VR.ATO.GEN.438001

                /* VR.ATO.GEN.438001
                Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPCollectablesExists == true && cgtsChildDocument.RPCollectablesEndDate != report.iitrParentDocument.RPEndDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438001",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period end date is incorrect.",
                        LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPCollectablesId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438001

                #region VR.ATO.GEN.438001

                /* VR.ATO.GEN.438001
                Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPOtherCapitalAssetsExists == true && cgtsChildDocument.RPOtherCapitalAssetsEndDate != report.iitrParentDocument.RPEndDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438001",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period end date is incorrect.",
                        LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPOtherCapitalAssetsId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438001

                #region VR.ATO.GEN.438001

                /* VR.ATO.GEN.438001
                Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPCollectablesUnappliedInstantInstantDate != null && cgtsChildDocument.RPCollectablesUnappliedInstantInstantDate != report.iitrParentDocument.RPEndDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438001",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period end date is incorrect.",
                        LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPCollectablesUnappliedInstantId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438001

                #region VR.ATO.GEN.438001

                /* VR.ATO.GEN.438001
                Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPCGTOtherUnappliedInstantInstantDate != null && cgtsChildDocument.RPCGTOtherUnappliedInstantInstantDate != report.iitrParentDocument.RPEndDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438001",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period end date is incorrect.",
                        LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPCGTOtherUnappliedInstantId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438001

                #region VR.ATO.GEN.438001

                /* VR.ATO.GEN.438001
                Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                Legacy Rule Format:
                IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                    RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (cgtsChildDocument.RPTransfersToOtherClassesExists == true && cgtsChildDocument.RPTransfersToOtherClassesEndDate != report.iitrParentDocument.RPEndDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438001",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"The context period end date is incorrect.",
                        LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsChildDocument.RPTransfersToOtherClassesId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438001

                #region VR.ATO.GEN.438022

                /* VR.ATO.GEN.438022
                If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Small business active asset reduction (Subdivision 152-C)", then Small business active asset reduction amount on attached CGT schedule must be completed. 

                Legacy Rule Format:
                IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "A - Small business active asset reduction") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS127] = NULL
                    RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                CGTS:RP:^CGTS127 = tns:TaxConcession.CapitalGains.SmallBusinessActiveAssetReduction.Amount
                IITR:RP:^IITR310 = tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code
                */
                assertion = ((report.iitrParentDocument.IITR310Collection != null ? report.iitrParentDocument.IITR310Collection.Contains("A - Small business active asset reduction") : false) && cgtsChildDocument.CGTS127 == null);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438022",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Small business active asset reduction amount missing from CGT schedule",
                        LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Small business active asset reduction (Subdivision 152-C)"", then Small business active asset reduction amount on attached CGT schedule must be completed",
                        Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438022" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS127", Value = cgtsChildDocument.CGTS127.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR310", Value = "A - Small business active asset reduction" });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438022

                #region VR.ATO.GEN.438023

                /* VR.ATO.GEN.438023
                If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Small business retirement exemption (Subdivision152-D)", then Small business retirement exemption amount on attached CGT schedule must be completed. 

                Legacy Rule Format:
                IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "B - Small business retirement exemption") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS128] = NULL
                    RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                CGTS:RP:^CGTS128 = tns:TaxConcession.CapitalGains.SmallBusinessRetirementExemption.Amount
                IITR:RP:^IITR310 = tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code
                */
                assertion = ((report.iitrParentDocument.IITR310Collection != null ? report.iitrParentDocument.IITR310Collection.Contains("B - Small business retirement exemption") : false) && cgtsChildDocument.CGTS128 == null);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438023",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Small business retirement exemption amount missing from CGT schedule",
                        LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Small business retirement exemption (Subdivision152-D)"", then Small business retirement exemption amount on attached CGT schedule must be completed",
                        Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438023" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS128", Value = cgtsChildDocument.CGTS128.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR310", Value = "B - Small business retirement exemption" });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438023

                #region VR.ATO.GEN.438024

                /* VR.ATO.GEN.438024
                If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Small business roll-over (Subdivision 152-E)", then Small business roll-over amount on attached CGT schedule must be completed. 

                Legacy Rule Format:
                IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "C - Small business roll-over") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS129] = NULL
                    RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                CGTS:RP:^CGTS129 = tns:TaxConcession.CapitalGains.SmallBusinessRollover.Amount
                IITR:RP:^IITR310 = tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code
                */
                assertion = ((report.iitrParentDocument.IITR310Collection != null ? report.iitrParentDocument.IITR310Collection.Contains("C - Small business roll-over") : false) && cgtsChildDocument.CGTS129 == null);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438024",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Small business roll-over amount missing from CGT schedule",
                        LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Small business roll-over (Subdivision 152-E)"", then Small business roll-over amount on attached CGT schedule must be completed",
                        Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438024" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS129", Value = cgtsChildDocument.CGTS129.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR310", Value = "C - Small business roll-over" });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438024

                #region VR.ATO.GEN.438025

                /* VR.ATO.GEN.438025
                If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Small business 15 year exemption (Subdivision152-B)", then Small business 15 year exemption amount on attached CGT schedule must be completed. 

                Legacy Rule Format:
                IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "D - Small business 15 year exemption") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS138] = NULL
                    RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                CGTS:RP:^CGTS138 = tns:TaxConcession.CapitalGains.SmallBusinessExemptionExempt.Amount
                IITR:RP:^IITR310 = tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code
                */
                assertion = ((report.iitrParentDocument.IITR310Collection != null ? report.iitrParentDocument.IITR310Collection.Contains("D - Small business 15 year exemption") : false) && cgtsChildDocument.CGTS138 == null);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438025",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Small business 15 year exemption amount missing from CGT schedule",
                        LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Small business 15 year exemption (Subdivision152-B)"", then Small business 15 year exemption amount on attached CGT schedule must be completed",
                        Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438025" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS138", Value = cgtsChildDocument.CGTS138.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR310", Value = "D - Small business 15 year exemption" });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438025

                #region VR.ATO.GEN.438026

                /* VR.ATO.GEN.438026
                If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Foreign resident CGT exemption (Division 855)", then Capital gains disregarded by a foreign resident amount on attached CGT schedule must be completed. 

                Legacy Rule Format:
                IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "E - Foreign resident CGT exemption") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS140] = NULL
                    RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                CGTS:RP:^CGTS140 = tns:Capital.Gains.ForeignResidentExemption.Amount
                IITR:RP:^IITR310 = tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code
                */
                assertion = ((report.iitrParentDocument.IITR310Collection != null ? report.iitrParentDocument.IITR310Collection.Contains("E - Foreign resident CGT exemption") : false) && cgtsChildDocument.CGTS140 == null);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438026",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Capital gains disregarded by a foreign resident amount missing from CGT schedule",
                        LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Foreign resident CGT exemption (Division 855)"", then Capital gains disregarded by a foreign resident amount on attached CGT schedule must be completed",
                        Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438026" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS140", Value = cgtsChildDocument.CGTS140.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR310", Value = "E - Foreign resident CGT exemption" });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438026

                #region VR.ATO.GEN.438027

                /* VR.ATO.GEN.438027
                If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is "Scrip for scrip roll-over (Subdivision 124-M)", then Capital gains disregarded as a result of a scrip for scrip roll-over amount on attached CGT schedule must be completed. 

                Legacy Rule Format:
                IF (RP:rvctc3.xx.xx:CapitalGainsTax.ExemptionOrRolloverApplied.Code = "F - Scrip for scrip roll-over") AND (COUNT(SCHEDULE = "CGTS") = 1) AND [CGTS45] = NULL
                    RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                CGTS:RP:^CGTS45  = tns:Capital.Gains.ScriptForScriptRolloverDeferral.Amount
                IITR:RP:^IITR310 = tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code
                */
                assertion = ((report.iitrParentDocument.IITR310Collection != null ? report.iitrParentDocument.IITR310Collection.Contains("F - Scrip for scrip roll-over") : false) && cgtsChildDocument.CGTS45 == null);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.GEN.438027",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Capital gains disregarded as a result of a scrip for scrip roll-over amount missing from CGT schedule",
                        LongDescription = @"If taxpayer is required to lodge a CGT schedule and CGT exemption or rollover applied code is ""Scrip for scrip roll-over (Subdivision 124-M)"", then Capital gains disregarded as a result of a scrip for scrip roll-over amount on attached CGT schedule must be completed",
                        Location = "/xbrli:xbrl/tns:CapitalGainsTax.ExemptionOrRolloverApplied.Code[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438027" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS45", Value = cgtsChildDocument.CGTS45.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR310", Value = "F - Scrip for scrip roll-over" });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.GEN.438027

                #region VR.ATO.CGTS.402165

                /* VR.ATO.CGTS.402165
                Total losses transferred in applied is only valid for a company

                Legacy Rule Format:
                IF (PARENT RETURN <> "CTR") AND ([CGTS122] <> NULL)  
                    RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                CGTS:RP.Applied:^CGTS122 = tns:Capital.Losses.TransferredTotal.Amount
                */
                assertion = (cgtsChildDocument.CGTS122 != null);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.CGTS.402165",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Total losses transferred in applied is only valid for a company",
                        Location = "/xbrli:xbrl/tns:Capital.Losses.TransferredTotal.Amount[@contextRef='" + cgtsChildDocument.RPAppliedId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.CGTS.402165" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS122", Value = cgtsChildDocument.CGTS122.GetValueOrDefault().ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.CGTS.402165

                #region VR.ATO.CGTS.402226

                /* VR.ATO.CGTS.402226
                If the Amendment indicator on a parent return is set to TRUE, then the Income year earnout right created and Amount to be amended cannot be supplied.

                Legacy Rule Format:
                IF (PARENT RETURN:RP:pyin.xx.xx:Report.Amendment.Indicator = TRUE) AND ([CGTS150] <> NULL AND [CGTS151] <> NULL)
                    RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                CGTS:RP.{EarnoutArrangementGroupNumber}:^CGTS150 = tns:Report.TargetFinancial.Year
                CGTS:RP.{EarnoutArrangementGroupNumber}:^CGTS151 = tns:Miscellaneous.RequestedAmendment.Amount
                IITR:RP:^IITR16                                  = tns:Report.Amendment.Indicator
                */
                assertion = (report.iitrParentDocument.IITR16 == true && cgtsChildDocument.CGTS150 != null && cgtsChildDocument.CGTS151 != null);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.CGTS.402226",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Income year earnout right created and Amount to be amended cannot be supplied when Parent return Amendment indicator is present.",
                        Location = "/xbrli:xbrl/tns:Report.TargetFinancial.Year[@contextRef='" + cgtsChildDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.CGTS.402226" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS150", Value = cgtsChildDocument.CGTS150.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS151", Value = cgtsChildDocument.CGTS151.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR16", Value = report.iitrParentDocument.IITR16.GetValueOrDefault().ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.CGTS.402226
                
                #region VR.ATO.CGTS.402230

                /* VR.ATO.CGTS.402230
                A multi-class treatment can only be reported with a Trust Tax Return for Attribution Managed Investments (TRTAMI) when a multi-class election has been made

                Legacy Rule Format:
                IF PARENT RETURN  <> "TRTAMI" AND ([CGTS153] <> NULL OR [CGTS154] <> NULL OR [CGTS155] <> NULL)
                    RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                CGTS:RP:^CGTS153                         = tns:IncomeTax.AttributionManagedInvestmentTrustClass.Description
                CGTS:RP.TransfersToOtherClasses:^CGTS154 = tns:Capital.Gains.Total.Amount
                CGTS:RP.TransfersToOtherClasses:^CGTS155 = tns:Capital.Losses.Total.Amount
                */
                assertion = (cgtsChildDocument.CGTS153 != null || cgtsChildDocument.CGTS154 != null || cgtsChildDocument.CGTS155 != null);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.CGTS.402230",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Multi-class treatment fields incorrectly supplied",
                        Location = "/xbrli:xbrl/tns:IncomeTax.AttributionManagedInvestmentTrustClass.Description[@contextRef='" + cgtsChildDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.CGTS.402230" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS153", Value = cgtsChildDocument.CGTS153 });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS154", Value = cgtsChildDocument.CGTS154.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS155", Value = cgtsChildDocument.CGTS155.GetValueOrDefault().ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.CGTS.402230

                #region VR.ATO.CGTS.402240

                /* VR.ATO.CGTS.402240
                The year entered at Income year earnout right created cannot be the current year.

                Legacy Rule Format:
                IF ([CGTS150] <> NULL) AND ([CGTS150] = (PARENT RETURN:RP:pyin.xx.xx:Report.TargetFinancial.Year))
                    RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                CGTS:RP.{EarnoutArrangementGroupNumber}:^CGTS150 = tns:Report.TargetFinancial.Year
                IITR:RP:^IITR10                                  = tns:Report.TargetFinancial.Year
                */
                assertion = (cgtsChildDocument.CGTS150 != null && cgtsChildDocument.CGTS150 == report.iitrParentDocument.IITR10);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.CGTS.402240",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Income year earnout right created incorrect",
                        LongDescription = @"The year entered at Income year earnout right created cannot be the current year.",
                        Location = "/xbrli:xbrl/tns:Report.TargetFinancial.Year[@contextRef='" + cgtsChildDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.CGTS.402240" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS150", Value = cgtsChildDocument.CGTS150.ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR10", Value = report.iitrParentDocument.IITR10.ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.CGTS.402240
                
                #region VR.ATO.IITR.310063

                /* VR.ATO.IITR.310063
                For an attached Capital Gains Tax schedule, if Net capital gain is greater than zero, there cannot be an amount at Other net capital losses carried forward to later income years.

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "CGTS") = 1) AND ([cgts125] > 0) AND ([cgts131] > 0)
                    RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                CGTS:RP.CGTOther.Unapplied(Instant):^CGTS125 = tns:Capital.Losses.CarriedForward.Net.Amount
                CGTS:RP:^CGTS131                             = tns:Income.CapitalGainsNet.Amount
                */
                assertion = (cgtsChildDocument.CGTS125.GetValueOrDefault() > 0 && cgtsChildDocument.CGTS131.GetValueOrDefault() > 0);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.310063",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Other net capital losses carried forward to later income years on CGTS schedule is invalid",
                        LongDescription = @"For an attached Capital Gains Tax (CGTS) schedule, when a 'Net capital gain' amount above zero is provided, 'Other net capital losses carried forward to later income years' must be zero or not provided",
                        Location = "/xbrli:xbrl/xbrli:context",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310063" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS125", Value = cgtsChildDocument.CGTS125.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS131", Value = cgtsChildDocument.CGTS131.GetValueOrDefault().ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310063

                #region VR.ATO.IITR.310068

                /* VR.ATO.IITR.310068
                The total current year capital gains on the Capital Gains Tax schedule must be equal to the total current year capital gains on the Individual return.

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "CGTS") = 1) AND ([iitr312] <> [cgts118])
                    RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                CGTS:RP:^CGTS118 = tns:Capital.Gains.Total.Amount
                IITR:RP:^IITR312 = tns:Capital.Gains.Total.Amount
                */
                assertion = (report.iitrParentDocument.IITR312.GetValueOrDefault() != cgtsChildDocument.CGTS118.GetValueOrDefault());
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.310068",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Total current year capital gains amount invalid",
                        LongDescription = @"The Total current year capital gains on the Capital Gains Tax schedule must be equal to the Total current year capital gains on the Individual return",
                        Location = "/xbrli:xbrl/tns:Capital.Gains.Total.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310068" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS118", Value = cgtsChildDocument.CGTS118.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR312", Value = report.iitrParentDocument.IITR312.GetValueOrDefault().ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310068

                #region VR.ATO.IITR.310070

                /* VR.ATO.IITR.310070
                The sum of unapplied net capital losses carried forward on the Capital Gains Tax schedule must equal net capital losses carried forward on the main income tax return.

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "CGTS") = 1) AND ([iitr313] <> CGTS:RP.Collectables.Unapplied:rvctc3.xx.xx:Capital.Losses.CarriedForward.Net.Amount + CGTS:RP.CGTOther.Unapplied:rvctc3.xx.xx:Capital.Losses.CarriedForward.Net.Amount)
                    RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                CGTS:RP.Collectables.Unapplied:^CGTS124 = tns:Capital.Losses.CarriedForward.Net.Amount
                CGTS:RP.CGTOther.Unapplied:^CGTS125     = tns:Capital.Losses.CarriedForward.Net.Amount
                IITR:RP(Instant):^IITR313               = tns:Capital.Losses.CarriedForward.Net.Amount
                */
                assertion = (report.iitrParentDocument.IITR313.GetValueOrDefault() != (cgtsChildDocument.CGTS124.GetValueOrDefault() + cgtsChildDocument.CGTS125.GetValueOrDefault()));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.310070",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Net capital losses carried forward incorrect",
                        LongDescription = @"The sum of unapplied net capital losses carried forward on the Capital Gains Tax schedule must equal Net capital losses carried forward to later income years on the main income tax return",
                        Location = "/xbrli:xbrl/tns:Capital.Losses.CarriedForward.Net.Amount[@contextRef='" + report.iitrParentDocument.RPInstantId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310070" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS124", Value = cgtsChildDocument.CGTS124.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS125", Value = cgtsChildDocument.CGTS125.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR313", Value = report.iitrParentDocument.IITR313.GetValueOrDefault().ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310070

                #region VR.ATO.IITR.310071

                /* VR.ATO.IITR.310071
                The net capital gain amount on the main return form must equal the net capital gain amount on the Capital Gains Tax schedule.

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "CGTS") = 1) AND ([iitr311] <> CGTS:RP:bafpr1.xx.xx:Income.CapitalGainsNet.Amount)
                    RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                CGTS:RP:^CGTS131 = tns:Income.CapitalGainsNet.Amount
                IITR:RP:^IITR311 = tns:Capital.Gains.Net.Amount
                */
                assertion = (report.iitrParentDocument.IITR311.GetValueOrDefault() != cgtsChildDocument.CGTS131.GetValueOrDefault());
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.310071",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Net capital gain amount in return incorrect",
                        LongDescription = @"The Net capital gain amount on the main return form must equal the Net capital gain amount on the Capital Gains Tax schedule",
                        Location = "/xbrli:xbrl/tns:Capital.Gains.Net.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310071" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS131", Value = cgtsChildDocument.CGTS131.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR311", Value = report.iitrParentDocument.IITR311.GetValueOrDefault().ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310071

                #region VR.ATO.IITR.310085

                /* VR.ATO.IITR.310085
                The Total CGT discount applied is greater than 50% of the Total current year capital gains less Total capital losses applied.

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "CGTS") = 1) AND ([CGTS126] > ((([CGTS118] - [CGTS123]) *0.5) +1))
                    RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                CGTS:RP:^CGTS118         = tns:Capital.Gains.Total.Amount
                CGTS:RP.Applied:^CGTS123 = tns:Capital.Losses.Total.Amount
                CGTS:RP.Applied:^CGTS126 = tns:TaxConcession.CapitalGains.DiscountTotal.Amount
                */
                assertion = (cgtsChildDocument.CGTS126.GetValueOrDefault() > (((cgtsChildDocument.CGTS118.GetValueOrDefault() - cgtsChildDocument.CGTS123.GetValueOrDefault()) * (decimal) 0.5) + 1));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.310085",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Total CGT discount applied amount incorrect",
                        LongDescription = @"The Total CGT discount applied amount must not exceed 50% of the combined amount of Total current year capital gains, less Total capital losses applied",
                        Location = "/xbrli:xbrl/xbrli:context",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310085" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS118", Value = cgtsChildDocument.CGTS118.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS123", Value = cgtsChildDocument.CGTS123.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "CGTS126", Value = cgtsChildDocument.CGTS126.GetValueOrDefault().ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310085

                #region Repeating RPEarnoutArrangementGroupNumberContext

                if (cgtsChildDocument.RPEarnoutArrangementGroupNumberContextCollection != null)
                {
                    foreach (CGTS2018.RPEarnoutArrangementGroupNumberContext cgtsEarnoutArrangementGroupNumberContext in cgtsChildDocument.RPEarnoutArrangementGroupNumberContextCollection)
                    {
                        #region VR.ATO.GEN.438000

                        /* VR.ATO.GEN.438000
                        Period start date within context on schedule doesn't match start date within PARENT RETURN:reporting party context

                        Legacy Rule Format:
                        IF (period.startDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.startDate
                            RETURN VALIDATION MESSAGE
                        ENDIF
                        */
                        assertion = (cgtsEarnoutArrangementGroupNumberContext.StartDate != report.iitrParentDocument.RPStartDate);
                        if (assertion)
                        {
                            processMessage = new ProcessMessageDocument()
                            {
                                Code = "CMN.ATO.GEN.438000",
                                Severity = ProcessMessageSeverity.Error,
                                Description = @"The context period start date is incorrect.",
                                LongDescription = @"Period start date within context on the schedule must match period start date within the parent return reporting party context",
                                Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsEarnoutArrangementGroupNumberContext.Id + "']",
                                Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438000" } },
                            };

                            response.Add(processMessage);
                        }

                        #endregion // VR.ATO.GEN.438000

                        #region VR.ATO.GEN.438001

                        /* VR.ATO.GEN.438001
                        Period end date within context on schedule doesn't match end date within PARENT RETURN:reporting party context

                        Legacy Rule Format:
                        IF (period.endDate WHERE CONTEXT(ALL)) <> PARENT RETURN:RP:period.endDate
                            RETURN VALIDATION MESSAGE
                        ENDIF
                        */
                        assertion = (cgtsEarnoutArrangementGroupNumberContext.EndDate != report.iitrParentDocument.RPEndDate);
                        if (assertion)
                        {
                            processMessage = new ProcessMessageDocument()
                            {
                                Code = "CMN.ATO.GEN.438001",
                                Severity = ProcessMessageSeverity.Error,
                                Description = @"The context period end date is incorrect.",
                                LongDescription = @"Period end date within context on the schedule must match period end date within the parent return reporting party context",
                                Location = "/xbrli:xbrl/xbrli:context[@id='" + cgtsEarnoutArrangementGroupNumberContext.Id + "']",
                                Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.438001" } },
                            };

                            response.Add(processMessage);
                        }

                        #endregion // VR.ATO.GEN.438001
                    }
                }

                #endregion // Repeating RPEarnoutArrangementGroupNumberContext
            }

            #endregion // CGTS Child Present

            // ####################################################################################################
            //                                         DIDS CROSSFORM RULES
            // ####################################################################################################

            #region VR.ATO.IITR.310100

            /*  VR.ATO.IITR.310100
            When the sum of Interest deductions and Dividend deductions exceeds $4,999, an Interest and Dividend deductions schedule must be attached. The sum of Amount claimed as interest or dividend expense on the Interest and Dividend deductions schedule must also equal the sum of Interest deductions and Dividend deductions on the parent form.
    
            Legacy Rule Format:
            IF (([iitr146] + [iitr147]) > 4999) AND ((COUNT(SCHEDULE = "DIDS") = 0) OR (([iitr146] + [iitr147]) <> SUM([dids21])))
			   RETURN VALIDATION MESSAGE
			ENDIF

            Data Elements:
			
			DIDS:RP:^DIDS21 = tns:IncomeTax.Deduction.InterestDividendClaimed.Amount
            IITR:RP:^IITR146 = tns:IncomeTax.Deduction.Interest.Amount
			IITR:RP:^IITR147 = tns:IncomeTax.Deduction.Dividend.Amount
            */
            assertion = ((report.iitrParentDocument.IITR146.GetValueOrDefault() + report.iitrParentDocument.IITR147.GetValueOrDefault()) > 4999 && ((report.didsChildDocuments == null || Count(report.didsChildDocuments) == 0) || (report.iitrParentDocument.IITR146.GetValueOrDefault() + report.iitrParentDocument.IITR147.GetValueOrDefault()) != report.didsChildDocuments.Sum(f => f.DIDS21.GetValueOrDefault())));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.310100",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Sum of Interest deductions and Dividend deductions is incorrect",
                    LongDescription = @"When the sum of Interest deductions and Dividend deductions exceeds $4,999, an Interest and Dividend deductions schedule must be attached. The sum of Amount claimed as interest or dividend expense on the Interest and Dividend deductions schedule must also equal the sum of Interest deductions and Dividend deductions on the parent form.",
                    Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.Interest.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310100" } },
                };
				processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "DIDS21", Value = (Count(report.didsChildDocuments) > 0 ? report.didsChildDocuments.Sum(f => f.DIDS21.GetValueOrDefault()).ToString() : "0") });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR146", Value = report.iitrParentDocument.IITR146.GetValueOrDefault().ToString() });
				processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR147", Value = report.iitrParentDocument.IITR147.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }

            #endregion // VR.ATO.IITR.310100

            #region DIDS Child Present

            if (report.didsChildDocuments != null && Count(report.didsChildDocuments) > 0)
            {
                foreach (DIDS2018 didsChildDocument in report.didsChildDocuments)
                {
                    #region VR.ATO.IITR.600040

                    /*  VR.ATO.IITR.600040
                    Invalid context. Period start date for the Interest and Dividends Deductions Schedule must be the same as the Period start date of the parent form.

                    Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "DIDS") > 0) AND (ANY OCCURRENCE OF (DIDS:RP:period.startDate) <> RP:period.startDate)
                        RETURN VALIDATION MESSAGE
                    ENDIF

                    Data Elements:

                    DIDS:RP:^StartDate = RP
                    IITR:RP:^StartDate = RP
                    */
                    assertion = (didsChildDocument.RPStartDate != report.iitrParentDocument.RPStartDate);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IITR.600040",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Period start date for the Interest and Dividend Deductions Schedule is invalid",
                            LongDescription = @"Invalid context. Period start date for the Interest and Dividends Deductions Schedule must be the same as the Period start date of the parent form.",
                            Location = "/xbrli:xbrl/xbrli:context[@id='" + didsChildDocument.RPId + "']/xbrli:period/xbrli:startDate",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.600040" } },
                        };

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.IITR.600040

                    #region VR.ATO.IITR.600041

                    /*  VR.ATO.IITR.600041
                    Invalid context. Period end date for the Interest and Dividends Deductions Schedule must be the same as the Period end date of the parent form.

                    Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "DIDS") > 0) AND (ANY OCCURRENCE OF (DIDS:RP:period.endDate) <> RP:period.endDate)
                        RETURN VALIDATION MESSAGE
                    ENDIF

                    Data Elements:

                    DIDS:RP:^EndDate = RP
                    IITR:RP:^EndDate = RP
                    */
                    assertion = (didsChildDocument.RPEndDate != report.iitrParentDocument.RPEndDate);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IITR.600041",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Period end date for the Interest and Dividend Deductions Schedule is invalid",
                            LongDescription = @"Invalid context. Period end date for the Interest and Dividends Deductions Schedule must be the same as the Period end date of the parent form.",
                            Location = "/xbrli:xbrl/xbrli:context[@id='" + didsChildDocument.RPId + "']/xbrli:period/xbrli:endDate",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.600041" } },
                        };

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.IITR.600041

                    #region VR.ATO.IITR.600042

                    /*  VR.ATO.IITR.600042
                    The TFN provided for the Interest and Dividend Deductions Schedule must be the same as the TFN of the parent form

                    Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "DIDS") > 0) AND (ANY OCCURRENCE OF (DIDS:RP:entity.identifier.TFN) <> RP:entity.identifier.TFN)
                       RETURN VALIDATION MESSAGE
                    ENDIF

                    Data Elements:

                    DIDS:RP:^EntityTFN = RP
                    IITR:RP:^EntityTFN = RP
                    */
                    assertion = (didsChildDocument.RPIdentifierTFN != report.iitrParentDocument.RPIdentifierTFN);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IITR.600042",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Tax file number invalid",
                            LongDescription = @"The TFN provided for the Interest and Dividend Deductions Schedule must be the same as the TFN of the parent form",
                            Location = "/xbrli:xbrl/xbrli:context[@id='" + didsChildDocument.RPId + "']/xbrli:entity/xbrli:identifier",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.600042" } },
                        };

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.IITR.600042
                }
            }
            #endregion // Repeating DIDS Child Documents

            // ####################################################################################################
            //                                         NRFI CROSSFORM RULES
            // ####################################################################################################

            #region VR.ATO.IITR.000639

            /* VR.ATO.IITR.000639
            An amount has been entered at Non-resident foreign income amount and there is no Non-Resident Foreign Income (NRFI) schedule attached. Please complete and attach a NRFI schedule.

            Legacy Rule Format:
            IF [iitr494] <> NULL AND COUNT(SCHEDULE = "NRFI") = 0
               RETURN VALIDATION MESSAGE
            ENDIF
            */
            assertion = (report.iitrParentDocument.IITR494 != null && (report.nrfiChildDocuments == null || Count(report.nrfiChildDocuments) == 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.000639",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Non-Resident Foreign Income (NRFI) schedule is required for Non-resident foreign income amount",
                    LongDescription = @"An amount has been entered at Non-resident foreign income amount and there is no Non-Resident Foreign Income (NRFI) schedule attached. Please complete and attach a NRFI schedule.",
                    Location = "/xbrli:xbrl/tns:Income.Net.Amount[@contextRef='" + report.iitrParentDocument.RPJFOREIGNId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.000639" } },
                };

                response.Add(processMessage);
            }

            #endregion // VR.ATO.IITR.000639

            #region VR.ATO.IITR.310093

            /* VR.ATO.IITR.310093
            The number of Non-Resident Foreign Income schedules must not exceed 1

            Legacy Rule Format:
            IF COUNT(SCHEDULE = "NRFI") > 1
                RETURN VALIDATION MESSAGE
            ENDIF
            */
            assertion = (report.nrfiChildDocuments != null && Count(report.nrfiChildDocuments) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.310093",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"The number of Non-Resident Foreign Income schedules must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310093" } },
                };

                response.Add(processMessage);
            }

            #endregion // VR.ATO.IITR.310093

            #region NRFI Child Present

            if (report.nrfiChildDocuments != null && Count(report.nrfiChildDocuments) == 1)
            {
                NRFI2018 nrfiChildDocument = report.nrfiChildDocuments[0];

                #region VR.ATO.IITR.000641

                /* VR.ATO.IITR.000641
                When the simple self-assessment method is selected on the Non-Resident Foreign Income (NRFI) schedule, the Gross foreign income amount, less standard occupation deductions must equal the Non-resident foreign income amount on the parent form.

                Legacy Rule Format:
                IF COUNT(SCHEDULE = "NRFI") = 1 AND [iitr494] <> NULL AND [nrfi3] = "Simple" AND SUBSTRING([nrfi2],1,4) = SET(DOMAIN(OCCUPATION PREFIX CODES)) AND [iitr494] <> ([nrfi4] - RoundDown([nrfi4] * ConditionalLookup(SUBSTRING([nrfi2],1,4), 'Occupation Prefix Codes', NULL, 'Deduction Median Rate', 0),0))
                   RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (report.iitrParentDocument.IITR494 != null && nrfiChildDocument.NRFI3 == "Simple" && DeductionMedianRates.deductionMedianRates.Keys.Contains(nrfiChildDocument.NRFI2.Substring(0, 4)) && report.iitrParentDocument.IITR494.GetValueOrDefault() != (nrfiChildDocument.NRFI4.GetValueOrDefault() - Math.Floor(nrfiChildDocument.NRFI4.GetValueOrDefault() * (Decimal) DeductionMedianRates.deductionMedianRates[nrfiChildDocument.NRFI2.Substring(0, 4)])));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.000641",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Non-resident foreign income amount using the simple self-assessment method is incorrect",
                        LongDescription = @"When the simple self-assessment method is selected on the Non-Resident Foreign Income (NRFI) schedule, the Gross foreign income amount, less standard occupation deductions must equal the Non-resident foreign income amount on the parent form",
                        Location = "/xbrli:xbrl/tns:Income.Net.Amount[@contextRef='" + report.iitrParentDocument.RPJFOREIGNId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.000641" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.000641

                #region VR.ATO.IITR.000642

                /* VR.ATO.IITR.000642
                When the overseas assessed method is selected on the Non-Resident Foreign Income (NRFI) schedule, the Foreign tax return taxable income must equal the Non-resident foreign income amount on the parent form, with the exception that when Foreign tax return taxable income on the NRFI schedule is negative, the Non-resident foreign income amount on the parent form must be zero (0).

                Legacy Rule Format:
                IF COUNT(SCHEDULE = "NRFI") = 1 AND [iitr494] <> NULL AND [nrfi3] = "Overseas" AND (([nrfi10] >= 0 AND [iitr494] <> [nrfi10]) OR ([nrfi10] < 0 AND [iitr494] <> 0))
                   RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (report.iitrParentDocument.IITR494 != null && nrfiChildDocument.NRFI3 == "Overseas" && ((nrfiChildDocument.NRFI10.GetValueOrDefault() >= 0 && report.iitrParentDocument.IITR494.GetValueOrDefault() != nrfiChildDocument.NRFI10.GetValueOrDefault()) || (nrfiChildDocument.NRFI10.GetValueOrDefault() < 0 && report.iitrParentDocument.IITR494.GetValueOrDefault() != 0)));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.000642",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Non-resident foreign income amount using the overseas assessed method is incorrect",
                        LongDescription = @"When the overseas assessed method is selected on the Non-Resident Foreign Income (NRFI) schedule, the Foreign tax return taxable income must equal the Non-resident foreign income amount on the parent form, with the exception that when Foreign tax return taxable income on the NRFI schedule is negative, the Non-resident foreign income amount on the parent form must be zero (0)",
                        Location = "/xbrli:xbrl/tns:Income.Net.Amount[@contextRef='" + report.iitrParentDocument.RPJFOREIGNId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.000642" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.000642

                #region VR.ATO.IITR.000643

                /* VR.ATO.IITR.000643
                When the comprehensive assessment method is selected on the Non-Resident Foreign Income (NRFI) schedule, the sum of Total salary or wages amount, Total allowance amount, Total government allowances amount, Total pension amount, Net business income, Total personal services income, Total interest amount, Total dividend amount, Net partnership and trust income amount, Net capital gains amount, Net rent amount and Total other income amount, less the sum of Total employee work related deduction amount, Total depreciation amount, Total interest and dividend amount, Total undeducted purchase price of a foreign pension amount, Total personal super contributions for business, Total other deduction amount and Prior year tax losses amount, must equal the Non-resident foreign income amount on the parent form, with the exception that when the sum of amounts on the NRFI schedule is negative, the Non-resident foreign income amount on the parent form must be zero (0).

                Legacy Rule Format:
                IF COUNT(SCHEDULE = "NRFI") = 1 AND [iitr494] <> NULL AND [nrfi3] = "Comprehensive" AND ((([nrfi11] + [nrfi12] + [nrfi13] + [nrfi14] + [nrfi15] + [nrfi16] + [nrfi17] + [nrfi18] + [nrfi19] + [nrfi20] + [nrfi21] + [nrfi22]) >= ([nrfi23] + [nrfi24] + [nrfi25] + [nrfi26] + [nrfi27] + [nrfi28] + [nrfi32]) AND [iitr494] <> (([nrfi11] + [nrfi12] + [nrfi13] + [nrfi14] + [nrfi15] + [nrfi16] + [nrfi17] + [nrfi18] + [nrfi19] + [nrfi20] + [nrfi21] + [nrfi22]) - ([nrfi23] + [nrfi24] + [nrfi25] + [nrfi26] + [nrfi27] + [nrfi28] + [nrfi32]))) OR (([nrfi11] + [nrfi12] + [nrfi13] + [nrfi14] + [nrfi15] + [nrfi16] + [nrfi17] + [nrfi18] + [nrfi19] + [nrfi20] + [nrfi21] + [nrfi22]) < ([nrfi23] + [nrfi24] + [nrfi25] + [nrfi26] + [nrfi27] + [nrfi28] + [nrfi32]) AND [iitr494] <> 0))
                   RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (report.iitrParentDocument.IITR494 != null && nrfiChildDocument.NRFI3 == "Comprehensive" && (((nrfiChildDocument.NRFI11.GetValueOrDefault() + nrfiChildDocument.NRFI12.GetValueOrDefault() + nrfiChildDocument.NRFI13.GetValueOrDefault() + nrfiChildDocument.NRFI14.GetValueOrDefault() + nrfiChildDocument.NRFI15.GetValueOrDefault() + nrfiChildDocument.NRFI16.GetValueOrDefault() + nrfiChildDocument.NRFI17.GetValueOrDefault() + nrfiChildDocument.NRFI18.GetValueOrDefault() + nrfiChildDocument.NRFI19.GetValueOrDefault() + nrfiChildDocument.NRFI20.GetValueOrDefault() + nrfiChildDocument.NRFI21.GetValueOrDefault() + nrfiChildDocument.NRFI22.GetValueOrDefault()) >= (nrfiChildDocument.NRFI23.GetValueOrDefault() + nrfiChildDocument.NRFI24.GetValueOrDefault() + nrfiChildDocument.NRFI25.GetValueOrDefault() + nrfiChildDocument.NRFI26.GetValueOrDefault() + nrfiChildDocument.NRFI27.GetValueOrDefault() + nrfiChildDocument.NRFI28.GetValueOrDefault() + nrfiChildDocument.NRFI32.GetValueOrDefault()) && report.iitrParentDocument.IITR494.GetValueOrDefault() != (nrfiChildDocument.NRFI11.GetValueOrDefault() + nrfiChildDocument.NRFI12.GetValueOrDefault() + nrfiChildDocument.NRFI13.GetValueOrDefault() + nrfiChildDocument.NRFI14.GetValueOrDefault() + nrfiChildDocument.NRFI15.GetValueOrDefault() + nrfiChildDocument.NRFI16.GetValueOrDefault() + nrfiChildDocument.NRFI17.GetValueOrDefault() + nrfiChildDocument.NRFI18.GetValueOrDefault() + nrfiChildDocument.NRFI19.GetValueOrDefault() + nrfiChildDocument.NRFI20.GetValueOrDefault() + nrfiChildDocument.NRFI21.GetValueOrDefault() + nrfiChildDocument.NRFI22.GetValueOrDefault()) - (nrfiChildDocument.NRFI23.GetValueOrDefault() + nrfiChildDocument.NRFI24.GetValueOrDefault() + nrfiChildDocument.NRFI25.GetValueOrDefault() + nrfiChildDocument.NRFI26.GetValueOrDefault() + nrfiChildDocument.NRFI27.GetValueOrDefault() + nrfiChildDocument.NRFI28.GetValueOrDefault() + nrfiChildDocument.NRFI32.GetValueOrDefault())) || ((nrfiChildDocument.NRFI11.GetValueOrDefault() + nrfiChildDocument.NRFI12.GetValueOrDefault() + nrfiChildDocument.NRFI13.GetValueOrDefault() + nrfiChildDocument.NRFI14.GetValueOrDefault() + nrfiChildDocument.NRFI15.GetValueOrDefault() + nrfiChildDocument.NRFI16.GetValueOrDefault() + nrfiChildDocument.NRFI17.GetValueOrDefault() + nrfiChildDocument.NRFI18.GetValueOrDefault() + nrfiChildDocument.NRFI19.GetValueOrDefault() + nrfiChildDocument.NRFI20.GetValueOrDefault() + nrfiChildDocument.NRFI21.GetValueOrDefault() + nrfiChildDocument.NRFI22.GetValueOrDefault()) < (nrfiChildDocument.NRFI23.GetValueOrDefault() + nrfiChildDocument.NRFI24.GetValueOrDefault() + nrfiChildDocument.NRFI25.GetValueOrDefault() + nrfiChildDocument.NRFI26.GetValueOrDefault() + nrfiChildDocument.NRFI27.GetValueOrDefault() + nrfiChildDocument.NRFI28.GetValueOrDefault() + nrfiChildDocument.NRFI32.GetValueOrDefault()) && report.iitrParentDocument.IITR494.GetValueOrDefault() != 0)));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.000643",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Non-resident foreign income amount using the comprehensive assessment method is incorrect",
                        LongDescription = @"When the comprehensive assessment method is selected on the Non-Resident Foreign Income (NRFI) schedule, the sum of Total salary or wages amount, Total allowance amount, Total government allowances amount, Total pension amount, Net business income, Total personal services income, Total interest amount, Total dividend amount, Net partnership and trust income amount, Net capital gains amount, Net rent amount and Total other income amount, less the sum of Total employee work related deduction amount, Total depreciation amount, Total interest and dividend amount, Total undeducted purchase price of a foreign pension amount, Total personal super contributions for business, Total other deduction amount and Prior year tax losses amount, must equal the Non-resident foreign income amount on the parent form, with the exception that when the sum of amounts on the NRFI schedule is negative, the Non-resident foreign income amount on the parent form must be zero (0)",
                        Location = "/xbrli:xbrl/tns:Income.Net.Amount[@contextRef='" + report.iitrParentDocument.RPJFOREIGNId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.000643" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.000643

                #region VR.ATO.IITR.000644

                /* VR.ATO.IITR.000644
                Foreign tax return financial year provided on Non-Resident Foreign Income (NRFI) schedule must be within one year of Year of return on parent form

                Legacy Rule Format:
                IF COUNT(SCHEDULE = "NRFI") = 1 AND [nrfi7] <> NULL AND [nrfi7] <> [iitr10] +/- 1
                   RETURN VALIDATION RULE
                ENDIF
                */
                assertion = (nrfiChildDocument.NRFI7 != null && OutsideRange((Decimal) nrfiChildDocument.NRFI7, (Decimal) report.iitrParentDocument.IITR10, 1));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.000644",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Foreign tax return financial year is invalid",
                        LongDescription = @"Foreign tax return financial year provided on Non-Resident Foreign Income (NRFI) schedule must be within one year of Year of return on parent form",
                        Location = "/xbrli:xbrl/tns:Report.TargetFinancial.Year[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.000644" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.000644

                #region VR.ATO.IITR.000645

                /* VR.ATO.IITR.000645
                Unable to confirm whether the correct Non-resident foreign income amount has been provided for the simple self-assessment method, as the Foreign occupation code provided on the Non-Resident Foreign Income (NRFI) schedule is not recognised. Record the details of the message and contact the agency service desk.

                Legacy Rule Format:
                IF COUNT(SCHEDULE = "NRFI") = 1 AND [nrfi3] = "Simple" AND [nrfi2] <> NULL AND SUBSTRING([nrfi2],1,4) <> SET(DOMAIN(OCCUPATION PREFIX CODES))
                   RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (nrfiChildDocument.NRFI3 == "Simple" && nrfiChildDocument.NRFI2 != null && !DeductionMedianRates.deductionMedianRates.Keys.Contains(nrfiChildDocument.NRFI2.Substring(0, 4)));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.000645",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Foreign occupation code is unknown",
                        LongDescription = @"Unable to calculate the correct Non-resident foreign income amount for the simple self-assessment method, as the Foreign occupation code provided on the Non-Resident Foreign Income (NRFI) schedule is not configured. Record the details of the message and contact the agency service desk.",
                        Location = "/xbrli:xbrl/tns:Income.Net.Amount[@contextRef='" + report.iitrParentDocument.RPJFOREIGNId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.000645" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.000645

                #region VR.ATO.IITR.000651

                /* VR.ATO.IITR.000651
                When a Non-Resident Foreign Income (NRFI) schedule is attached, the Non-resident foreign income amount must be provided

                Legacy Rule Format:
                IF COUNT(SCHEDULE = "NRFI") = 1 AND [iitr494] = NULL
                   RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (report.iitrParentDocument.IITR494 == null);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.000651",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Non-resident foreign income amount must be provided",
                        LongDescription = @"When a Non-Resident Foreign Income (NRFI) schedule is attached, the Non-resident foreign income amount must be provided",
                        Location = "/xbrli:xbrl/tns:Income.Net.Amount[@contextRef='" + report.iitrParentDocument.RPJFOREIGNId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.000651" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.000651

                #region VR.ATO.IITR.310094

                /* VR.ATO.IITR.310094
                Invalid context. Period start date for the Non-Resident Foreign Income Schedule must be the same as the Period start date of the parent form.

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "NRFI") = 1) AND (NRFI:RP:period.startDate <> RP:period.startDate)
                   RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (nrfiChildDocument.RPStartDate != report.iitrParentDocument.RPStartDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.310094",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Period start date for the Non-Resident Foreign Income Schedule is invalid",
                        LongDescription = @"Invalid context. Period start date for the Non-Resident Foreign Income Schedule must be the same as the Period start date of the parent form.",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + nrfiChildDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310094" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310094

                #region VR.ATO.IITR.310095

                /* VR.ATO.IITR.310095
                Invalid context. Period end date for the Non-Resident Foreign Income Schedule must be the same as the Period end date of the parent form.

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "NRFI") = 1) AND (NRFI:RP:period.endDate <> RP:period.endDate)
                   RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (nrfiChildDocument.RPEndDate != report.iitrParentDocument.RPEndDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.310095",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Period end date for the Non-Resident Foreign Income Schedule is invalid",
                        LongDescription = @"Invalid context. Period end date for the Non-Resident Foreign Income Schedule must be the same as the Period end date of the parent form.",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + nrfiChildDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310095" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310095

                #region VR.ATO.IITR.600046

                /* VR.ATO.IITR.600046
                The TFN provided for the Non-Resident Foreign Income schedule must be the same as the TFN of the parent form

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "NRFI") = 1) AND (NRFI:RP:entity.identifier.TFN <> RP:entity.identifier.TFN)
                   RETURN VALIDATION MESSAGE
                ENDIF
                */
                assertion = (nrfiChildDocument.RPIdentifierTFN != report.iitrParentDocument.RPIdentifierTFN);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.600046",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Tax file number invalid",
                        LongDescription = @"The TFN provided for the Non-Resident Foreign Income schedule must be the same as the TFN of the parent form",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + nrfiChildDocument.RPId + "']/xbrli:entity/xbrli:identifier",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.600046" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.600046
            }

            #endregion // NRFI Child Present

            // ####################################################################################################
            //                                         PSS CROSSFORM RULES
            // ####################################################################################################

            #region VR.ATO.IITR.300041

            /*  VR.ATO.IITR.300041
            When an amount has been provided for 'Net income/loss from business tax withheld voluntary agreement', 'Net income/loss from business tax withheld where Australian business number not quoted', 'Net income/loss from business tax withheld - foreign resident withholding (excluding capital gains)', or 'Net income/loss from business tax withheld - labour hire or other specified payments', a Payment Summary Schedule (PSS) must be attached

            Legacy Rule Format:
            IF (([iitr286] > 0 OR [iitr287] > 0 OR [iitr288] > 0 OR [iitr289] > 0) AND COUNT(SCHEDULE = "PSS") = 0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Data Elements:

            IITR:RP:^IITR286 = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessVoluntaryAgreement.Amount
            IITR:RP:^IITR287 = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessABNNotQuoted.Amount
            IITR:RP:^IITR288 = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessForeignResident.Amount
            IITR:RP:^IITR289 = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessLabourHireOrOtherPayments.Amount
            */
            assertion = ((report.iitrParentDocument.IITR286.GetValueOrDefault() > 0 || report.iitrParentDocument.IITR287.GetValueOrDefault() > 0 || report.iitrParentDocument.IITR288.GetValueOrDefault() > 0 || report.iitrParentDocument.IITR289.GetValueOrDefault() > 0) && (report.pssChildDocuments == null || Count(report.pssChildDocuments) == 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.301041",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Payment Summary schedule is required for Business income tax withheld",
                    LongDescription = @"When an amount has been provided for 'Net income/loss from business tax withheld voluntary agreement', 'Net income/loss from business tax withheld where Australian business number not quoted', 'Net income/loss from business tax withheld - foreign resident withholding (excluding capital gains)', or 'Net income/loss from business tax withheld - labour hire or other specified payments', a Payment Summary Schedule (PSS) must be attached",
                    Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessVoluntaryAgreement.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300041" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR286", Value = report.iitrParentDocument.IITR286.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR287", Value = report.iitrParentDocument.IITR287.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR288", Value = report.iitrParentDocument.IITR288.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR289", Value = report.iitrParentDocument.IITR289.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }

            #endregion // VR.ATO.IITR.300041

            #region VR.ATO.IITR.300046

            /*  VR.ATO.IITR.300046
            An amount has been entered at Attributed personal services income and there is no Payment Summary schedule attached. Please complete and attach a Payment summary schedule for Individuals.

            Legacy Rule Format:
            IF ([iitr112] > 0 AND COUNT(SCHEDULE = "PSS" = 0))
               RETURN VALIDATION MESSAGE
            ENDIF

            Data Elements:

            IITR:RP:^IITR112 = tns:Income.PersonalServicesIncome.Total.Amount
            */
            assertion = (report.iitrParentDocument.IITR112.GetValueOrDefault() > 0 && (report.pssChildDocuments == null || Count(report.pssChildDocuments) == 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.300046",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Payment Summary schedule is required for an Attributed PSI",
                    LongDescription = @"An amount has been entered at Attributed personal services income and there is no Payment Summary schedule attached. Please complete and attach a Payment summary schedule for Individuals.",
                    Location = "/xbrli:xbrl/tns:Income.PersonalServicesIncome.Total.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300046" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR112", Value = report.iitrParentDocument.IITR112.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }

            #endregion // VR.ATO.IITR.300046

            #region VR.ATO.IITR.300047

            /*  VR.ATO.IITR.300047
            Amounts have been entered at Personal services income for the tax withheld amounts or PSI income and there is no Payment Summary schedule attached. Please complete and attach a Payment Summary schedule for Individuals.

            Legacy Rule Format:
            IF (([iitr276] > 0 OR [iitr277] > 0 OR [iitr278] > 0 OR [iitr411] > 0 OR [iitr412] > 0 OR [iitr413] > 0) AND COUNT(SCHEDULE = "PSS") = 0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Data Elements:

            IITR:RP:^IITR276 = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldPersonalServicesIncomeVoluntaryAgreement.Amount
            IITR:RP:^IITR277 = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldPersonalServicesIncomeABNNotQuoted.Amount
            IITR:RP:^IITR278 = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldPersonalServicesIncomeLabourHireOrOtherPayments.Amount
            IITR:RP:^IITR411 = tns:Income.PersonalServicesIncome.VoluntaryAgreement.Amount
            IITR:RP:^IITR412 = tns:Income.PersonalServicesIncome.ABNNotQuoted.Amount
            IITR:RP:^IITR413 = tns:Income.PersonalServicesIncome.LabourHireOrOtherSpecifiedPayment.Amount
            */
            assertion = ((report.iitrParentDocument.IITR276.GetValueOrDefault() > 0 || report.iitrParentDocument.IITR277.GetValueOrDefault() > 0 || report.iitrParentDocument.IITR278.GetValueOrDefault() > 0 || report.iitrParentDocument.IITR411.GetValueOrDefault() > 0 || report.iitrParentDocument.IITR412.GetValueOrDefault() > 0 || report.iitrParentDocument.IITR413.GetValueOrDefault() > 0) && (report.pssChildDocuments == null || Count(report.pssChildDocuments) == 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.300047",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Payment Summary schedule is required for Personal services income",
                    LongDescription = @"Amounts have been entered at Personal services income for the tax withheld amounts or PSI income and there is no Payment Summary schedule attached. Please complete and attach a Payment Summary schedule for Individuals.",
                    Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldPersonalServicesIncomeVoluntaryAgreement.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300047" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR276", Value = report.iitrParentDocument.IITR276.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR277", Value = report.iitrParentDocument.IITR277.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR278", Value = report.iitrParentDocument.IITR278.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR411", Value = report.iitrParentDocument.IITR411.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR412", Value = report.iitrParentDocument.IITR412.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR413", Value = report.iitrParentDocument.IITR413.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }

            #endregion // VR.ATO.IITR.300047
            
            #region VR.ATO.IITR.310021

            /*  VR.ATO.IITR.310021
            Amounts have been entered at primary production - Gross payments or non-primary production - Gross payments and there is no Payment Summary schedule attached. A Payment summary schedule for Individuals must be completed

            Legacy Rule Format:
            IF (([iitr436] + [iitr437] + [iitr438] + [iitr439] + [iitr440] + [iitr441] + [iitr442]) > 0 AND COUNT(SCHEDULE = "PSS") = 0)
               RETURN VALIDATION MESSAGE
            ENDIF

            Data Elements:

            IITR:RP:^IITR436 = tns:Income.ABNNotQuotedPrimaryProductionPaymentGross.Amount
            IITR:RP:^IITR437 = tns:Income.ABNNotQuotedNonPrimaryProductionPaymentGross.Amount
            IITR:RP:^IITR438 = tns:Income.ForeignResidentWithholdingNonPrimaryProduction.Amount
            IITR:RP:^IITR439 = tns:Income.VoluntaryAgreementPrimaryProductionGross.Amount
            IITR:RP:^IITR440 = tns:Income.VoluntaryAgreementNonPrimaryProductionGross.Amount
            IITR:RP:^IITR441 = tns:Income.LabourHireArrangementPrimaryProductionPaymentGross.Amount
            IITR:RP:^IITR442 = tns:Income.LabourHireArrangementNonPrimaryProductionPaymentGross.Amount
            */
            assertion = ((report.iitrParentDocument.IITR436.GetValueOrDefault() + report.iitrParentDocument.IITR437.GetValueOrDefault() + report.iitrParentDocument.IITR438.GetValueOrDefault() + report.iitrParentDocument.IITR439.GetValueOrDefault() + report.iitrParentDocument.IITR440.GetValueOrDefault() + report.iitrParentDocument.IITR441.GetValueOrDefault() + report.iitrParentDocument.IITR442.GetValueOrDefault()) > 0 && (report.pssChildDocuments == null || Count(report.pssChildDocuments) == 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.310021",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Payment Summary schedule is required for business income",
                    LongDescription = @"Amounts have been entered at primary production - Gross payments or non-primary production - Gross payments and there is no Payment Summary schedule attached. A Payment summary schedule for Individuals must be completed.",
                    Location = "/xbrli:xbrl/tns:Income.ABNNotQuotedPrimaryProductionPaymentGross.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310021" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR436", Value = report.iitrParentDocument.IITR436.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR437", Value = report.iitrParentDocument.IITR437.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR438", Value = report.iitrParentDocument.IITR438.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR439", Value = report.iitrParentDocument.IITR439.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR440", Value = report.iitrParentDocument.IITR440.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR441", Value = report.iitrParentDocument.IITR441.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR442", Value = report.iitrParentDocument.IITR442.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }

            #endregion // VR.ATO.IITR.310021

            #region VR.ATO.IITR.310089

            /*  VR.ATO.IITR.310089
            The number of Payment Summary schedules must not exceed 1.

            Legacy Rule Format:
            IF COUNT(SCHEDULE = "PSS") > 1
                RETURN VALIDATION MESSAGE
            ENDIF
            */
            assertion = (report.pssChildDocuments != null && Count(report.pssChildDocuments) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.310089",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"The number of Payment Summary schedules must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310089" } },
                };

                response.Add(processMessage);
            }

            #endregion // VR.ATO.IITR.310089

            #region PSS Child Present

            if (report.pssChildDocuments != null && Count(report.pssChildDocuments) == 1)
            {
                PSS2018 pssChildDocument = report.pssChildDocuments[0];

                #region VR.ATO.IITR.300042

                /*  VR.ATO.IITR.300042
                The sum of Tax withheld where ABN not quoted amount in the Payment summary schedule, where Income Type is set to Business, does not equal Net income/loss from business tax withheld where Australian business number not quoted on the main return form

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "PSS") = 1) AND (SUM([pss23] WHERE [pss15] = "Business")) <> [iitr287]
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR287               = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessABNNotQuoted.Amount
                PSS:RP.Payer.{PSSeqNum}:^PSS15 = tns:IncomeTax.IncomeType.Code
                PSS:RP.Payer.{PSSeqNum}:^PSS23 = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessABNNotQuoted.Amount
                */
                assertion = (pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Business").Sum(context => context.PSS23.GetValueOrDefault()) != report.iitrParentDocument.IITR287.GetValueOrDefault());
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.300042",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Tax withheld where ABN not quoted amount is incorrect",
                        LongDescription = @"The sum of Tax withheld where ABN not quoted amount in the Payment summary schedule, where Income Type is set to Business, does not equal Net income/loss from business tax withheld where Australian business number not quoted on the main return form",
                        Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessABNNotQuoted.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300042" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR287", Value = report.iitrParentDocument.IITR287.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS15", Value = "Business" });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS23", Value = pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Business").Sum(context => context.PSS23.GetValueOrDefault()).ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.300042

                #region VR.ATO.IITR.300043

                /*  VR.ATO.IITR.300043
                The sum of Tax withheld for Voluntary agreement withholding amount in the Payment summary schedule, where the Income Type is set to Business, does not equal Net income/loss from business tax withheld voluntary agreement on the main tax return.

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "PSS") = 1) AND (SUM([pss24] WHERE [pss15] = "Business")) <> [iitr286]
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR286               = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessVoluntaryAgreement.Amount
                PSS:RP.Payer.{PSSeqNum}:^PSS15 = tns:IncomeTax.IncomeType.Code
                PSS:RP.Payer.{PSSeqNum}:^PSS24 = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessVoluntaryAgreement.Amount
                */
                assertion = (pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Business").Sum(context => context.PSS24.GetValueOrDefault()) != report.iitrParentDocument.IITR286.GetValueOrDefault());
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.300043",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Tax withheld - voluntary agreement amount is incorrect",
                        LongDescription = @"The sum of Tax withheld for Voluntary agreement withholding amount in the Payment summary schedule, where the Income Type is set to Business, does not equal Net income/loss from business tax withheld voluntary agreement on the main tax return",
                        Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessVoluntaryAgreement.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300043" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR286", Value = report.iitrParentDocument.IITR286.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS15", Value = "Business" });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS24", Value = pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Business").Sum(context => context.PSS24.GetValueOrDefault()).ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.300043

                #region VR.ATO.IITR.300044

                /*  VR.ATO.IITR.300044
                The sum of Tax withheld for labour hire or other specified payments withholding amount in the Payment summary schedule, where Income Type is set to Business, does not equal Net income/loss from business tax withheld - labour hire or other specified payments on the main return form

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "PSS") = 1) AND (SUM([pss25] WHERE [pss15] = "Business")) <> [iitr289]
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR289               = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessLabourHireOrOtherPayments.Amount
                PSS:RP.Payer.{PSSeqNum}:^PSS15 = tns:IncomeTax.IncomeType.Code
                PSS:RP.Payer.{PSSeqNum}:^PSS25 = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessLabourHireOrOtherPayments.Amount
                */
                assertion = (pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Business").Sum(context => context.PSS25.GetValueOrDefault()) != report.iitrParentDocument.IITR289.GetValueOrDefault());
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.300044",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Tax withheld - labour hire or other payments amount is incorrect",
                        LongDescription = @"The sum of Tax withheld for labour hire or other specified payments withholding amount in the Payment summary schedule, where Income Type is set to Business, does not equal Net income/loss from business tax withheld - labour hire or other specified payments on the main return form",
                        Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessLabourHireOrOtherPayments.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300044" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR289", Value = report.iitrParentDocument.IITR289.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS15", Value = "Business" });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS25", Value = pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Business").Sum(context => context.PSS25.GetValueOrDefault()).ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.300044

                #region VR.ATO.IITR.300045

                /*  VR.ATO.IITR.300045
                The sum of 'Tax withheld for foreign resident withholding' in the Payment Summary Schedule (PSS), where the associated Income Type is either Business or Personal Services Income, must equal the 'Net income/loss from business tax withheld - foreign resident withholding (excluding capital gains)' amount on the main return form

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "PSS") = 1) AND (SUM([pss22] WHERE [pss15] = SET("Business", "Personal Services Income"))) <> [iitr288]
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR288               = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessForeignResident.Amount
                PSS:RP.Payer.{PSSeqNum}:^PSS15 = tns:IncomeTax.IncomeType.Code
                PSS:RP.Payer.{PSSeqNum}:^PSS22 = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessForeignResident.Amount
                */
                assertion = (pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Business" || context.PSS15 == "Personal Services Income").Sum(context => context.PSS22.GetValueOrDefault()) != report.iitrParentDocument.IITR288.GetValueOrDefault());
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.301045",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Net income/loss from business tax withheld - foreign resident withholding (excluding capital gains) amount is incorrect",
                        LongDescription = @"The sum of 'Tax withheld for foreign resident withholding' in the Payment Summary Schedule (PSS), where the associated Income Type is either Business or Personal Services Income, must equal the 'Net income/loss from business tax withheld - foreign resident withholding (excluding capital gains)' amount on the main return form",
                        Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessForeignResident.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300045" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR288", Value = report.iitrParentDocument.IITR288.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS15", Value = pssChildDocument.RPPayerPSSeqNumContextCollection.Find(context => context.PSS15 == "Business" || context.PSS15 == "Personal Services Income").PSS15 });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS22", Value = pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Business").Sum(context => context.PSS22.GetValueOrDefault()).ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.300045

                #region VR.ATO.IITR.300048

                /*  VR.ATO.IITR.300048
                The amount shown at Tax withheld - voluntary agreement is not equal to the sum of the amounts of Tax withheld for Voluntary agreement withholding on the Payment Summary schedule where Income Type is set to Personal Service Income

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "PSS") = 1) AND (([iitr276] <> NULL AND [iitr276] <> SUM([pss24] WHERE [pss15] = "Personal Services Income")) OR ([iitr276] = NULL AND SUM([pss24] WHERE [pss15] = "Personal Services Income") <> 0))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR276               = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldPersonalServicesIncomeVoluntaryAgreement.Amount
                PSS:RP.Payer.{PSSeqNum}:^PSS15 = tns:IncomeTax.IncomeType.Code
                PSS:RP.Payer.{PSSeqNum}:^PSS24 = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessVoluntaryAgreement.Amount
                */
                assertion = ((report.iitrParentDocument.IITR276 != null && report.iitrParentDocument.IITR276 != pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS24.GetValueOrDefault())) || (report.iitrParentDocument.IITR276 == null && pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS24.GetValueOrDefault()) != 0));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.300048",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"PSI tax withheld - voluntary agreement is incorrect",
                        LongDescription = @"The amount shown at Tax withheld - voluntary agreement is not equal to the sum of the amounts of Tax withheld for Voluntary agreement withholding on the Payment Summary schedule where Income Type is set to Personal Service Income",
                        Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldPersonalServicesIncomeVoluntaryAgreement.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300048" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR276", Value = report.iitrParentDocument.IITR276.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS15", Value = "Personal Services Income" });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS24", Value = pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS24).ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.300048

                #region VR.ATO.IITR.300049

                /*  VR.ATO.IITR.300049
                The amount shown at Tax withheld where Australian business number not quoted is not equal to the sum of Tax withheld for withholding where ABN not quoted on the Payment Summary schedule where Income Type is set to Personal Service Income

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "PSS") = 1) AND (([iitr277] <> NULL AND [iitr277] <> SUM([pss23] WHERE [pss15] = "Personal Services Income")) OR ([iitr277] = NULL AND SUM([pss23] WHERE [pss15] = "Personal Services Income") <> 0))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR277               = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldPersonalServicesIncomeABNNotQuoted.Amount
                PSS:RP.Payer.{PSSeqNum}:^PSS15 = tns:IncomeTax.IncomeType.Code
                PSS:RP.Payer.{PSSeqNum}:^PSS23 = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessABNNotQuoted.Amount
                */
                assertion = ((report.iitrParentDocument.IITR277 != null && report.iitrParentDocument.IITR277 != pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS23.GetValueOrDefault())) || (report.iitrParentDocument.IITR277 == null && pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS23.GetValueOrDefault()) != 0));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.300049",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"PSI tax withheld where ABN not quoted is incorrect",
                        LongDescription = @"The amount shown at Tax withheld where Australian business number not quoted is not equal to the sum of Tax withheld for withholding where ABN not quoted on the Payment Summary schedule where Income Type is set to Personal Service Income",
                        Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldPersonalServicesIncomeABNNotQuoted.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300049" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR277", Value = report.iitrParentDocument.IITR277.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS15", Value = "Personal Services Income" });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS23", Value = pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS23).ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.300049

                #region VR.ATO.IITR.300050

                /*  VR.ATO.IITR.300050
                The amount shown at Personal Services Income tax withheld - labour hire and other specified payments is not equal to the sum of Tax withheld for labour hire or other specified payments withholding on the Payment Summary schedule where Income Type is set to Personal Service Income

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "PSS") = 1) AND (([iitr278] <> NULL AND [iitr278] <> SUM([pss25] WHERE [pss15] = "Personal Services Income")) OR ([iitr278] = NULL AND SUM([pss25] WHERE [pss15] = "Personal Services Income") <> 0))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR278               = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldPersonalServicesIncomeLabourHireOrOtherPayments.Amount
                PSS:RP.Payer.{PSSeqNum}:^PSS15 = tns:IncomeTax.IncomeType.Code
                PSS:RP.Payer.{PSSeqNum}:^PSS25 = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldBusinessLabourHireOrOtherPayments.Amount
                */
                assertion = ((report.iitrParentDocument.IITR278 != null && report.iitrParentDocument.IITR278 != pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS25.GetValueOrDefault())) || (report.iitrParentDocument.IITR278 == null && pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS25.GetValueOrDefault()) != 0));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.300050",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"PSI tax withheld - labour hire and other specified payments is incorrect",
                        LongDescription = @"The amount shown at Personal Services Income tax withheld - labour hire and other specified payments is not equal to the sum of Tax withheld for labour hire or other specified payments withholding on the Payment Summary schedule where Income Type is set to Personal Service Income",
                        Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldPersonalServicesIncomeLabourHireOrOtherPayments.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300050" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR278", Value = report.iitrParentDocument.IITR278.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS15", Value = "Personal Services Income" });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS25", Value = pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS25.GetValueOrDefault()).ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.300050

                #region VR.ATO.IITR.300051

                /*  VR.ATO.IITR.300051
                The amount shown at Tax withheld from attributed personal services income is not equal to the sum of Tax withheld for Attributed Personal Services Income on the Payment Summary schedule where Income Type is set to Personal Service Income

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "PSS") = 1) AND (([iitr111] <> NULL AND [iitr111] <> (SUM([pss32] WHERE [pss15] = "Personal Services Income")) +/- 1) OR ([iitr111] = NULL AND (SUM([pss32] WHERE [pss15] = "Personal Services Income")) +/- 1 <> 0))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR111               = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldPersonalServicesIncome.Amount
                PSS:RP.Payer.{PSSeqNum}:^PSS15 = tns:IncomeTax.IncomeType.Code
                PSS:RP.Payer.{PSSeqNum}:^PSS32 = tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldPersonalServicesIncome.Amount
                */
                assertion = ((report.iitrParentDocument.IITR111 != null && OutsideRange(report.iitrParentDocument.IITR111.GetValueOrDefault(), pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS32.GetValueOrDefault()), 1)) || (report.iitrParentDocument.IITR111 == null && OutsideRange(0, pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS32.GetValueOrDefault()), 1)));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.300051",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Tax withheld from attributed personal services income is incorrect",
                        LongDescription = @"The amount shown at Tax withheld from attributed personal services income is not equal to the sum of Tax withheld for Attributed Personal Services Income on the Payment Summary schedule where Income Type is set to Personal Service Income",
                        Location = "/xbrli:xbrl/tns:IncomeTax.PayAsYouGoWithholding.CreditTaxWithheldPersonalServicesIncome.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300051" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR111", Value = report.iitrParentDocument.IITR111.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS15", Value = "Personal Services Income" });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS32", Value = pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS32.GetValueOrDefault()).ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.300051

                #region VR.ATO.IITR.300052

                /*  VR.ATO.IITR.300052
                The amount shown at Attributed personal services income is not equal to the sum of Gross payment for Attributed Personal Services Income on the Payment Summary schedule where Income Type is set to Gross payment for Attributed Personal Services Income

                Legacy Rule Format:
                IF COUNT(SCHEDULE = "PSS") = 1 AND (([iitr112] <> NULL AND [iitr112] <> SUM([pss31] WHERE [pss15] = "Personal Services Income")) OR ([iitr112] = NULL AND SUM([pss31] WHERE [pss15] = "Personal Services Income") <> 0))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR112               = tns:Income.PersonalServicesIncome.Total.Amount
                PSS:RP.Payer.{PSSeqNum}:^PSS15 = tns:IncomeTax.IncomeType.Code
                PSS:RP.Payer.{PSSeqNum}:^PSS31 = tns:Remuneration.PersonalServicesIncome.AttributedGross.Amount
                */
                assertion = ((report.iitrParentDocument.IITR112 != null && report.iitrParentDocument.IITR112.GetValueOrDefault() != pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS31.GetValueOrDefault())) || (report.iitrParentDocument.IITR112 == null && pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS31.GetValueOrDefault()) != 0));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.300052",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Attributed personal services income is incorrect",
                        LongDescription = @"The amount shown at Attributed personal services income is not equal to the sum of Gross payment for Attributed Personal Services Income on the Payment Summary schedule where Income Type is set to Gross payment for Attributed Personal Services Income",
                        Location = "/xbrli:xbrl/tns:Income.PersonalServicesIncome.Total.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300052" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR112", Value = report.iitrParentDocument.IITR112.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS15", Value = "Personal Services Income" });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS32", Value = pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS31.GetValueOrDefault()).ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.300052

                #region VR.ATO.IITR.310035

                /*  VR.ATO.IITR.310035
                The sum of Gross payment for withholding where ABN not quoted in the payment summary schedule where Income Type is set to Business, should be equal to the sum of PP - Gross payments where Australian Business Number not quoted and Non-PP - Gross payments where Australian Business Number not quoted on the main return form

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "PSS") = 1) AND (SUM([pss19] WHERE [pss15] = "Business")) <> ([iitr436] + [iitr437])
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR436               = tns:Income.ABNNotQuotedPrimaryProductionPaymentGross.Amount
                IITR:RP:^IITR437               = tns:Income.ABNNotQuotedNonPrimaryProductionPaymentGross.Amount
                PSS:RP.Payer.{PSSeqNum}:^PSS15 = tns:IncomeTax.IncomeType.Code
                PSS:RP.Payer.{PSSeqNum}:^PSS19 = tns:Remuneration.ABNNotQuotedPaymentGross.Amount
                */
                assertion = (pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Business").Sum(context => context.PSS19.GetValueOrDefault()) != (report.iitrParentDocument.IITR436.GetValueOrDefault() + report.iitrParentDocument.IITR437.GetValueOrDefault()));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.310035",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Gross payment where ABN not quoted is incorrect",
                        LongDescription = @"The sum of Gross payment for withholding where ABN not quoted in the payment summary schedule where Income Type is set to Business, should be equal to the sum of PP - Gross payments where Australian Business Number not quoted and Non-PP - Gross payments where Australian Business Number not quoted on the main return form",
                        Location = "/xbrli:xbrl/tns:Income.ABNNotQuotedPrimaryProductionPaymentGross.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310035" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR436", Value = report.iitrParentDocument.IITR436.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR437", Value = report.iitrParentDocument.IITR437.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS15", Value = "Business" });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS19", Value = pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Business").Sum(context => context.PSS19.GetValueOrDefault()).ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310035

                #region VR.ATO.IITR.310036

                /*  VR.ATO.IITR.310036
                The sum of Gross payment for Voluntary agreement withholding in the payment summary schedule where Income Type is set to Business, should be equal to the sum of PP -  Gross payments voluntary agreement and Non-PP - Gross payments voluntary agreement on the main return form

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "PSS") = 1) AND (SUM([pss20] WHERE [pss15] = "Business")) <> ([iitr439] + [iitr440])
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR439               = tns:Income.VoluntaryAgreementPrimaryProductionGross.Amount
                IITR:RP:^IITR440               = tns:Income.VoluntaryAgreementNonPrimaryProductionGross.Amount
                PSS:RP.Payer.{PSSeqNum}:^PSS15 = tns:IncomeTax.IncomeType.Code
                PSS:RP.Payer.{PSSeqNum}:^PSS20 = tns:Remuneration.VoluntaryAgreementGross.Amount
                */
                assertion = (pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Business").Sum(context => context.PSS20.GetValueOrDefault()) != (report.iitrParentDocument.IITR439.GetValueOrDefault() + report.iitrParentDocument.IITR440.GetValueOrDefault()));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.310036",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Gross payments - voluntary agreement is incorrect",
                        LongDescription = @"The sum of Gross payment for Voluntary agreement withholding in the payment summary schedule where Income Type is set to Business, should be equal to the sum of PP -  Gross payments voluntary agreement and Non-PP - Gross payments voluntary agreement on the main return form",
                        Location = "/xbrli:xbrl/tns:Income.VoluntaryAgreementPrimaryProductionGross.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310036" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR439", Value = report.iitrParentDocument.IITR439.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR440", Value = report.iitrParentDocument.IITR440.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS15", Value = "Business" });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS20", Value = pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Business").Sum(context => context.PSS20.GetValueOrDefault()).ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310036

                #region VR.ATO.IITR.310037

                /*  VR.ATO.IITR.310037
                The sum of Gross payment for Labour hire or other specified payments withholding in the payment summary schedule where Income Type is set to Business, should be equal to the sum of PP - Gross payments labour hire or other specified payments and Non-PP - Gross payments labour hire or other specified payments on the main return form

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "PSS") = 1) AND (SUM([pss21] WHERE [pss15] = "Business")) <> ([iitr441] + [iitr442])
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR441               = tns:Income.LabourHireArrangementPrimaryProductionPaymentGross.Amount
                IITR:RP:^IITR442               = tns:Income.LabourHireArrangementNonPrimaryProductionPaymentGross.Amount
                PSS:RP.Payer.{PSSeqNum}:^PSS15 = tns:IncomeTax.IncomeType.Code
                PSS:RP.Payer.{PSSeqNum}:^PSS21 = tns:Remuneration.LabourHireArrangementPaymentGross.Amount
                */
                assertion = (pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Business").Sum(context => context.PSS21.GetValueOrDefault()) != (report.iitrParentDocument.IITR441.GetValueOrDefault() + report.iitrParentDocument.IITR442.GetValueOrDefault()));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.310037",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Gross payments - labour hire or other payments is incorrect",
                        LongDescription = @"The sum of Gross payment for Labour hire or other specified payments withholding in the payment summary schedule where Income Type is set to Business, should be equal to the sum of PP - Gross payments labour hire or other specified payments and Non-PP - Gross payments labour hire or other specified payments on the main return form",
                        Location = "/xbrli:xbrl/tns:Income.LabourHireArrangementPrimaryProductionPaymentGross.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310037" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR441", Value = report.iitrParentDocument.IITR441.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR442", Value = report.iitrParentDocument.IITR442.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS15", Value = "Business" });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS21", Value = pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Business").Sum(context => context.PSS21.GetValueOrDefault()).ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310037

                #region VR.ATO.IITR.310038

                /*  VR.ATO.IITR.310038
                The sum of Gross payment for foreign resident withholding in the Payment Summary schedule where Income Type is set to Business or Personal Services Income, should be equal to the Non-PP - Gross payments subject to foreign resident withholding (excluding capital gains) on the main return form

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "PSS") = 1) AND (SUM([pss18] WHERE [pss15] = SET("Business", "Personal Services Income")) <> [iitr438])
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR438               = tns:Income.ForeignResidentWithholdingNonPrimaryProduction.Amount
                PSS:RP.Payer.{PSSeqNum}:^PSS15 = tns:IncomeTax.IncomeType.Code
                PSS:RP.Payer.{PSSeqNum}:^PSS18 = tns:Remuneration.PaymentToForeignResidentGross.Amount
                */
                assertion = (pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Business" || context.PSS15 == "Personal Services Income").Sum(context => context.PSS18.GetValueOrDefault()) != report.iitrParentDocument.IITR438.GetValueOrDefault());
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.311038",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Gross payment - Foreign Resident Withholding is incorrect",
                        LongDescription = @"The sum of Gross payment for foreign resident withholding in the Payment Summary schedule where Income Type is set to Business or Personal Services Income, should be equal to the Non-PP - Gross payments subject to foreign resident withholding (excluding capital gains) on the main return form",
                        Location = "/xbrli:xbrl/tns:Income.ForeignResidentWithholdingNonPrimaryProduction.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310038" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR438", Value = report.iitrParentDocument.IITR438.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS18", Value = pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Business" || context.PSS15 == "Personal Services Income").Sum(context => context.PSS18.GetValueOrDefault()).ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310038

                #region VR.ATO.IITR.310039

                /*  VR.ATO.IITR.310039
                The amount shown at PSI - voluntary agreement is not equal to the sum of Gross payment for Voluntary agreement withholding on the Payment Summary schedule where Income Type is set to Personal Service Income

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "PSS") = 1) AND ((([iitr411] <> NULL) AND [iitr411] <> (SUM([pss20] WHERE [pss15] = "Personal Services Income"))) OR (([iitr411] = NULL) AND (SUM([pss20] WHERE [pss15] = "Personal Services Income") <> 0)))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR411               = tns:Income.PersonalServicesIncome.VoluntaryAgreement.Amount
                PSS:RP.Payer.{PSSeqNum}:^PSS15 = tns:IncomeTax.IncomeType.Code
                PSS:RP.Payer.{PSSeqNum}:^PSS20 = tns:Remuneration.VoluntaryAgreementGross.Amount
                */
                assertion = ((report.iitrParentDocument.IITR411 != null && report.iitrParentDocument.IITR411 != pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS20.GetValueOrDefault())) || (report.iitrParentDocument.IITR411 == null && pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS20.GetValueOrDefault()) != 0));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.310039",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"PSI - voluntary agreement is incorrect",
                        LongDescription = @"The amount shown at PSI - voluntary agreement is not equal to the sum of Gross payment for Voluntary agreement withholding on the Payment Summary schedule where Income Type is set to Personal Service Income",
                        Location = "/xbrli:xbrl/tns:Income.PersonalServicesIncome.VoluntaryAgreement.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310039" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR411", Value = report.iitrParentDocument.IITR411.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS15", Value = "Personal Services Income" });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS20", Value = pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS20.GetValueOrDefault()).ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310039

                #region VR.ATO.IITR.310040

                /*  VR.ATO.IITR.310040
                The amount shown at PSI where Australian business number not quoted is not equal to the sum of Gross payment for withholding where ABN not quoted on the Payment Summary schedule where Income Type is set to Personal Service Income

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "PSS") = 1) AND ((([iitr412] <> NULL) AND [iitr412] <> (SUM([pss19] WHERE [pss15] = "Personal Services Income"))) OR (([iitr412] = NULL) AND (SUM([pss19] WHERE [pss15] = "Personal Services Income") <> 0)))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR412               = tns:Income.PersonalServicesIncome.ABNNotQuoted.Amount
                PSS:RP.Payer.{PSSeqNum}:^PSS15 = tns:IncomeTax.IncomeType.Code
                PSS:RP.Payer.{PSSeqNum}:^PSS19 = tns:Remuneration.ABNNotQuotedPaymentGross.Amount
                */
                assertion = ((report.iitrParentDocument.IITR412 != null && report.iitrParentDocument.IITR412 != pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS19.GetValueOrDefault())) || (report.iitrParentDocument.IITR412 == null && pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS19.GetValueOrDefault()) != 0));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.310040",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"PSI where ABN not quoted is incorrect",
                        LongDescription = @"The amount shown at PSI where Australian business number not quoted is not equal to the sum of Gross payment for withholding where ABN not quoted on the Payment Summary schedule where Income Type is set to Personal Service Income",
                        Location = "/xbrli:xbrl/tns:Income.PersonalServicesIncome.ABNNotQuoted.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310040" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR412", Value = report.iitrParentDocument.IITR412.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS15", Value = "Personal Services Income" });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS19", Value = pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS19.GetValueOrDefault()).ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310040

                #region VR.ATO.IITR.310041

                /*  VR.ATO.IITR.310041
                The amount shown at PSI - labour hire or other specified payments is not equal to the sum of Gross payment for Labour hire or other specified payments withholding on the Payment Summary schedule where Income Type is set to Personal Service Income

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "PSS") = 1) AND ((([iitr413] <> NULL) AND [iitr413] <> (SUM([pss21] WHERE [pss15] = "Personal Services Income"))) OR (([iitr413] = NULL) AND (SUM([pss21] WHERE [pss15] = "Personal Services Income") <> 0)))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR413               = tns:Income.PersonalServicesIncome.LabourHireOrOtherSpecifiedPayment.Amount
                PSS:RP.Payer.{PSSeqNum}:^PSS15 = tns:IncomeTax.IncomeType.Code
                PSS:RP.Payer.{PSSeqNum}:^PSS21 = tns:Remuneration.LabourHireArrangementPaymentGross.Amount
                */
                assertion = ((report.iitrParentDocument.IITR413 != null && report.iitrParentDocument.IITR413 != pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS21.GetValueOrDefault())) || (report.iitrParentDocument.IITR413 == null && pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS21.GetValueOrDefault()) != 0));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.310041",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"PSI - labour hire and other specified payments is incorrect",
                        LongDescription = @"The amount shown at PSI - labour hire or other specified payments is not equal to the sum of Gross payment for Labour hire or other specified payments withholding on the Payment Summary schedule where Income Type is set to Personal Service Income",
                        Location = "/xbrli:xbrl/tns:Income.PersonalServicesIncome.LabourHireOrOtherSpecifiedPayment.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310041" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR413", Value = report.iitrParentDocument.IITR413.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS15", Value = "Personal Services Income" });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "PSS21", Value = pssChildDocument.RPPayerPSSeqNumContextCollection.FindAll(context => context.PSS15 == "Personal Services Income").Sum(context => context.PSS21.GetValueOrDefault()).ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310041
                
                #region Repeating RPPayerPSSeqNumContext

                if (pssChildDocument.RPPayerPSSeqNumContextCollection != null)
                {
                    foreach (PSS2018.RPPayerPSSeqNumContext pssPayerContext in pssChildDocument.RPPayerPSSeqNumContextCollection)
                    {
                        #region VR.ATO.IITR.310088

                        /*  VR.ATO.IITR.310088
                        When a Payment summary schedule for individuals is attached, the 'Income Type: Business or Personal Services Business' codes on the schedule must be provided.

                        Legacy Rule Format:
                        IF COUNT(SCHEDULE = "PSS") = 1 AND (ANY OCCURRENCE OF [pss15] = NULL)
                           RETURN VALIDATION MESSAGE
                        ENDIF

                        Data Elements:

                        PSS:RP.Payer.{PSSeqNum}:^PSS15 = tns:IncomeTax.IncomeType.Code
                        */
                        assertion = (pssPayerContext.PSS15 == null);
                        if (assertion)
                        {
                            processMessage = new ProcessMessageDocument()
                            {
                                Code = "CMN.ATO.IITR.310088",
                                Severity = ProcessMessageSeverity.Error,
                                Description = @"Income Type: Business or Personal Services is required",
                                LongDescription = @"When a Payment summary schedule for individuals is attached, the 'Income Type: Business or Personal Services Business' codes on the schedule must be provided",
                                Location = "/xbrli:xbrl/tns:IncomeTax.IncomeType.Code[@contextRef='" + pssPayerContext.Id + "']",
                                Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310088" } },
                            };
                            processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "PSS15", Value = "null" });

                            response.Add(processMessage);
                        }

                        #endregion // VR.ATO.IITR.310088

                        #region VR.ATO.IITR.400059

                        /*  VR.ATO.IITR.400059
                        Invalid context. Reporting Party Payer period start date must be the same as the Reporting Party Period start date.

                        Legacy Rule Format:
                        IF (COUNT(SCHEDULE = "PSS") = 1) AND (RP:period.startDate <> PSS:RP.Payer.{PSSeqNum}:period.StartDate)
                            RETURN VALIDATION MESSAGE
                        ENDIF

                        Data Elements:

                        IITR:RP:^StartDate                 = RP
                        PSS:RP.Payer.{PSSeqNum}:^StartDate = RP
                        */
                        assertion = (pssPayerContext.StartDate != report.iitrParentDocument.RPStartDate);
                        if (assertion)
                        {
                            processMessage = new ProcessMessageDocument()
                            {
                                Code = "CMN.ATO.IITR.400059",
                                Severity = ProcessMessageSeverity.Error,
                                Description = @"Reporting Party Payer Period start date is invalid",
                                LongDescription = @"Invalid context. Reporting Party Payer period start date must be the same as the Reporting Party Period start date.",
                                Location = "/xbrli:xbrl/xbrli:context[@id='" + pssPayerContext.Id + "']/xbrli:period/xbrli:startDate",
                                Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.400059" } },
                            };

                            response.Add(processMessage);
                        }

                        #endregion // VR.ATO.IITR.400059

                        #region VR.ATO.IITR.400060

                        /*  VR.ATO.IITR.400060
                        Invalid context. Reporting Party Payer period end date must be the same as the Reporting Party Period end date.

                        Legacy Rule Format:
                        IF (COUNT(SCHEDULE = "PSS") = 1) AND (RP:period.startDate <> PSS:RP.Payer.{PSSeqNum}:period.StartDate)
                            RETURN VALIDATION MESSAGE
                        ENDIF

                        Data Elements:

                        IITR:RP:^EndDate                 = RP
                        PSS:RP.Payer.{PSSeqNum}:^EndDate = RP
                        */
                        assertion = (pssPayerContext.EndDate != report.iitrParentDocument.RPEndDate);
                        if (assertion)
                        {
                            processMessage = new ProcessMessageDocument()
                            {
                                Code = "CMN.ATO.IITR.400060",
                                Severity = ProcessMessageSeverity.Error,
                                Description = @"Reporting Party Payer Period end date is invalid",
                                LongDescription = @"Invalid context. Reporting Party Payer period end date must be the same as the Reporting Party Period end date.",
                                Location = "/xbrli:xbrl/xbrli:context[@id='" + pssPayerContext.Id + "']/xbrli:period/xbrli:endDate",
                                Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.400060" } },
                            };

                            response.Add(processMessage);
                        }

                        #endregion // VR.ATO.IITR.400060

                        #region VR.ATO.PSS.000036

                        /*  VR.ATO.PSS.000036
                        Your supplied TFN does not match the TFN supplied on the form it was submitted with

                        Legacy Rule Format:
                        WHERE IN CONTEXT (RP.Payer.{PSSeqNum})
                        IF (RP.Payer.{PSSeqNum}.entity.identifier.TFN <> PARENT RETURN:RP.entity.identifier.TFN)
                           RETURN VALIDATION MESSAGE
                        ENDIF

                        Data Elements:

                        IITR:RP:^EntityTFN                 = RP
                        PSS:RP.Payer.{PSSeqNum}:^EntityTFN = RP
                        */
                        assertion = (pssPayerContext.IdentifierTFN != report.iitrParentDocument.RPIdentifierTFN);
                        if (assertion)
                        {
                            processMessage = new ProcessMessageDocument()
                            {
                                Code = "CMN.ATO.PSS.000036",
                                Severity = ProcessMessageSeverity.Error,
                                Description = @"Your supplied TFN does not match the TFN supplied on the form it was submitted with",
                                Location = "/xbrli:xbrl/xbrli:context[@id='" + pssPayerContext.Id + "']/xbrli:entity/xbrli:identifier",
                                Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.PSS.000036" } },
                            };

                            response.Add(processMessage);
                        }

                        #endregion // VR.ATO.PSS.000036
                    }
                }

                #endregion // Repeating RPPayerPSSeqNumContext
            }

            #endregion // PSS Child Present

            // ####################################################################################################
            //                                          RS CROSSFORM RULES
            // ####################################################################################################

            #region VR.ATO.IITR.210049

            /*  VR.ATO.IITR.210049
            For each Rental Schedule submitted with this parent return, the only valid Entity Type is 117

            Legacy Rule Format:
            IF COUNT(SCHEDULE = "RS") > 0 AND ANY OCCURRENCE OF [RS12] <> "117"
                RETURN VALIDATION MESSAGE
            ENDIF

            Data Elements:

            RS:RP:^RS12 = pyde.02.21:Party.Type.Code
            */
            assertion = (report.rsChildDocuments != null && report.rsChildDocuments.Count > 0 && report.rsChildDocuments.Any(doc => doc.RS12 != "117"));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.210049",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"For each Rental Schedule submitted with this parent return, the only valid Entity Type is 117",
                    Location = "/xbrli:xbrl/tns:Party.Type.Code[@contextRef='" + report.rsChildDocuments.Find(doc => doc.RS12 != "117").RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.210049" } },
                };

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IITR.210049

            #region VR.ATO.IITR.310083

            /*  VR.ATO.IITR.310083
            If there is a positive amount at Gross rent or at any Rental deductions item, then a Rental property schedule must be submitted for each rental property

            Legacy Rule Format:
            IF ([iitr334] > 0 OR [iitr335] > 0 OR [iitr336] > 0 OR [iitr337] > 0) AND (COUNT(SCHEDULE = "RS") = 0)
                RETURN VALIDATION MESSAGE
            ENDIF

            Data Elements:

            IITR:RP:^IITR334 = tns:Income.RealEstateProperty.Rental.Amount
            IITR:RP:^IITR335 = tns:IncomeTax.Deduction.RentalIncomeInterestDeductions.Amount
            IITR:RP:^IITR336 = tns:IncomeTax.Deduction.CapitalWorksDeductions.Amount
            IITR:RP:^IITR337 = tns:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount
            */
            assertion = ((report.iitrParentDocument.IITR334 > 0 || report.iitrParentDocument.IITR335 > 0 || report.iitrParentDocument.IITR336 > 0 || report.iitrParentDocument.IITR337 > 0) && (report.rsChildDocuments == null || Count(report.rsChildDocuments) == 0));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430081",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Rental schedule must be present",
                    LongDescription = "If there is a positive amount at Gross rent or at any Rental deductions item, then a Rental property schedule must be submitted for each rental property",
                    Location = "/xbrli:xbrl/tns:Income.RealEstateProperty.Rental.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310083" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR334", Value = report.iitrParentDocument.IITR334.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR335", Value = report.iitrParentDocument.IITR335.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR336", Value = report.iitrParentDocument.IITR336.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR337", Value = report.iitrParentDocument.IITR337.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IITR.310083

            #region VR.ATO.IITR.310084

            /*  VR.ATO.IITR.310084
            A rental schedule is attached AND rental income AND deduction amounts are not present in the return form. 

            Legacy Rule Format:
            IF (COUNT(SCHEDULE = "RS") > 0) AND ([iitr334] = NULL AND ([iitr335] = NULL OR [iitr335] = 0) AND ([iitr336] = NULL OR [iitr336] = 0) AND ([iitr337] = NULL OR [iitr337] = 0)) 
                RETURN VALIDATION MESSAGE
            ENDIF

            Data Elements:

            IITR:RP:^IITR334 = tns:Income.RealEstateProperty.Rental.Amount
            IITR:RP:^IITR335 = tns:IncomeTax.Deduction.RentalIncomeInterestDeductions.Amount
            IITR:RP:^IITR336 = tns:IncomeTax.Deduction.CapitalWorksDeductions.Amount
            IITR:RP:^IITR337 = tns:IncomeTax.Deduction.RentalIncomeDeductionsOtherThanInterestAndCapitalWorks.Amount
            */
            assertion = (report.rsChildDocuments != null && report.rsChildDocuments.Count > 0 && (report.iitrParentDocument.IITR334 == null && (report.iitrParentDocument.IITR335 == null || report.iitrParentDocument.IITR335 == 0) && (report.iitrParentDocument.IITR336 == null || report.iitrParentDocument.IITR336 == 0) && (report.iitrParentDocument.IITR337 == null || report.iitrParentDocument.IITR337 == 0)));
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.GEN.430082",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"Gross rent and Rental deduction amounts must be present",
                    LongDescription = "If a Rental property schedule is present, Gross rent or rental deductions must be present",
                    Location = "/xbrli:xbrl/tns:Income.RealEstateProperty.Rental.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310084" } },
                };
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR334", Value = report.iitrParentDocument.IITR334.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR335", Value = report.iitrParentDocument.IITR335.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR336", Value = report.iitrParentDocument.IITR336.GetValueOrDefault().ToString() });
                processMessage.Parameters.Add(new ProcessMessageParameter
                { Name = "IITR337", Value = report.iitrParentDocument.IITR337.GetValueOrDefault().ToString() });

                response.Add(processMessage);
            }
            #endregion // VR.ATO.IITR.310084

            #region RS Child Present

            if (report.rsChildDocuments != null && Count(report.rsChildDocuments) > 0)
            {
                foreach (RS2018 rsChildDocument in report.rsChildDocuments)
                {
                    #region VR.ATO.GEN.402009

                    /*  VR.ATO.GEN.402009
                    The Entity Identifier on the Schedule must equal the Entity Identifier on the Parent Return

                    Legacy Rule Format:
                    IF (RP:entity.identifier.TFN <> PARENT RETURN:RP:entity.identifier.TFN)
                        RETURN VALIDATION MESSAGE
                    ENDIF

                    Data Elements:

                    IITR:RP:^EntityTFN = RP
                    RS:RP:^EntityTFN   = RP
                    */
                    assertion = (rsChildDocument.RPIdentifierTFN != report.iitrParentDocument.RPIdentifierTFN);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.GEN.402009",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Your supplied TFN does not match the TFN supplied on the form it was submitted with",
                            LongDescription = @"Your supplied TFN does not match the TFN supplied on the parent form it was submitted with",
                            Location = "/xbrli:xbrl/xbrli:context[@id='" + rsChildDocument.RPId + "']/xbrli:entity/xbrli:identifier",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.GEN.402009" } },
                        };

                        response.Add(processMessage);
                    }

                    #endregion // VR.ATO.GEN.402009
            
                    #region VR.ATO.IITR.300153

                    /*  VR.ATO.IITR.300153
                    Invalid context. Period end date for the Rental schedule must be the same as the Period end date of the parent form.

                    Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "RS") > 0) AND (ANY OCCURRENCE OF (RS:RP:period.endDate) <> RP:period.endDate)
                        RETURN VALIDATION MESSAGE
                    ENDIF

                    Data Elements:

                    IITR:RP:^EndDate = RP
                    RS:RP:^EndDate   = RP
                    */
                    assertion = (rsChildDocument.RPEndDate != report.iitrParentDocument.RPEndDate);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IITR.300153",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Period end date for the Rental schedule is invalid",
                            LongDescription = @"Invalid context. Period end date for the Rental schedule must be the same as the Period end date of the parent form.",
                            Location = "/xbrli:xbrl/xbrli:context[@id='" + rsChildDocument.RPId + "']/xbrli:period/xbrli:endDate",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300153" } },
                        };

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.IITR.300153

                    #region VR.ATO.IITR.300155

                    /*  VR.ATO.IITR.300153
                    Invalid context. Period start date for the Rental schedule must be the same as the Period start date of the parent form.

                    Legacy Rule Format:
                    IF (COUNT(SCHEDULE = "RS") > 0) AND (ANY OCCURRENCE OF (RS:RP:period.startDate) <> RP:period.startDate)
                        RETURN VALIDATION MESSAGE
                    ENDIF

                    Data Elements:

                    IITR:RP:^StartDate = RP
                    RS:RP:^StartDate   = RP
                    */
                    assertion = (rsChildDocument.RPStartDate != report.iitrParentDocument.RPStartDate);
                    if (assertion)
                    {
                        processMessage = new ProcessMessageDocument()
                        {
                            Code = "CMN.ATO.IITR.300155",
                            Severity = ProcessMessageSeverity.Error,
                            Description = @"Period start date for the Rental schedule is invalid",
                            LongDescription = @"Invalid context. Period start date for the Rental schedule must be the same as the Period start date of the parent form.",
                            Location = "/xbrli:xbrl/xbrli:context[@id='" + rsChildDocument.RPId + "']/xbrli:period/xbrli:startDate",
                            Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300155" } },
                        };

                        response.Add(processMessage);
                    }
                    #endregion // VR.ATO.IITR.300155
                }
            }

            #endregion // Repeating RS Child Documents

            // ####################################################################################################
            //                                         WRE CROSSFORM RULES
            // ####################################################################################################

            #region VR.ATO.IITR.310086

            /* VR.ATO.IITR.310086
            The number of Work Related Expense schedules must not exceed 1.

            Legacy Rule Format:
            IF COUNT(SCHEDULE="WRE") > 1
               RETURN VALIDATION MESSAGE
            ENDIF

            */
            assertion = (report.wreChildDocuments != null && Count(report.wreChildDocuments) > 1);
            if (assertion)
            {
                processMessage = new ProcessMessageDocument()
                {
                    Code = "CMN.ATO.IITR.310086",
                    Severity = ProcessMessageSeverity.Error,
                    Description = @"The number of Work Related Expense schedules must not exceed 1",
                    Location = "/xbrli:xbrl/xbrli:context[@id='" + report.wreChildDocuments[1].RPId + "']",
                    Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310086" } },
                };

                response.Add(processMessage);
            }

            #endregion // VR.ATO.IITR.310086

            #region WRE Child Present

            if (report.wreChildDocuments != null && Count(report.wreChildDocuments) == 1)
            {
                WRE2017 wreChildDocument = report.wreChildDocuments[0];

                #region VR.ATO.IITR.300059

                /* VR.ATO.IITR.300059
                A schedule of work related expenses is attached and other travel expenses type is 'Award' and allowances, earnings, tips etc is missing or equal to zero in the individual return.

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "WRE") = 1) AND (ANY OCCURRENCE OF([wre25]) = "Award") AND (([iitr65] = NULL) OR ([iitr65] = 0))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^IITR65            = tns:Income.AllowancesEarningsTipsDirectorsFees.Amount
                WRE:RP.{TrvlSeqNum}:^WRE25 = tns:IncomeTax.Deduction.TravelCategory.Code
                */
                assertion = ((Count(wreChildDocument.RPTrvlSeqNumContextCollection) > 0 ? wreChildDocument.RPTrvlSeqNumContextCollection.Any(child => child.WRE25 == "Award") : false) && (report.iitrParentDocument.IITR65 == null || report.iitrParentDocument.IITR65.GetValueOrDefault() == 0));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.300059",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Amount of allowances, earnings, tips, director's fees, etc is required",
                        LongDescription = @"When ""Award"" is selected for 'Other travel expenses - Category of expense' on the attached Work related expenses schedule, an amount above zero must be provided for 'Amount of allowances, earnings, tips, director's fees, etc'",
                        Location = "/xbrli:xbrl/tns:Income.AllowancesEarningsTipsDirectorsFees.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300059" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR65", Value = report.iitrParentDocument.IITR65.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE25", Value = "Award" });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.300059

                #region VR.ATO.IITR.300079

                /* VR.ATO.IITR.300079
                A schedule of work related expenses is attached and other work expenses type is 'Overtime meal allowance' and allowances, earnings, tips, etc is missing or equal to zero in the individual return form.

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "WRE") = 1) AND (ANY OCCURRENCE OF([wre50]) = "Overtime meal allowance") AND (([iitr65] = NULL) OR ([iitr65] = 0))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                IITR:RP:^IITR65             = tns:Income.AllowancesEarningsTipsDirectorsFees.Amount
                WRE:RP.{OTHWRSeqNum}:^WRE50 = tns:IncomeTax.Deduction.WorkRelatedCategoryOther.Code
                */
                assertion = ((Count(wreChildDocument.RPOTHWRSeqNumContextCollection) > 0 ? wreChildDocument.RPOTHWRSeqNumContextCollection.Any(context => context.WRE50 == "Overtime meal allowance") : false) && (report.iitrParentDocument.IITR65 == null || report.iitrParentDocument.IITR65.GetValueOrDefault() == 0));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.300079",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Amount of allowances, earnings, tips, director's fees, etc is required",
                        LongDescription = @"When ""Overtime meal allowance"" is selected for 'Other work expenses - Category of expense' on the attached Work related expenses schedule, an amount above zero must be provided for 'Amount of allowances, earnings, tips, director's fees, etc'",
                        Location = "/xbrli:xbrl/tns:Income.AllowancesEarningsTipsDirectorsFees.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300079" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR65", Value = report.iitrParentDocument.IITR65.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE50", Value = "Overtime meal allowance" });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.300079

                #region VR.ATO.IITR.310101

                /* VR.ATO.IITR.310101
                A Schedule of Work Related Expenses is attached and the amount of self-education net deduction claimed is not equal to the amount of work related self-education expenses claimed in the Individual return.

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "WRE") = 1) AND ((([iitr139] <> NULL) AND ([iitr139] <> [wre47])) OR (([iitr139] = NULL) AND ([wre47] <> 0)))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                IITR:RP:^IITR139 = tns:IncomeTax.Deduction.EducationNet.Amount
                WRE:RP:^WRE47    = tns:IncomeTax.Deduction.EducationNet.Amount
                */
                assertion = (((report.iitrParentDocument.IITR139 != null && report.iitrParentDocument.IITR139.GetValueOrDefault() != wreChildDocument.WRE47.GetValueOrDefault()) || (report.iitrParentDocument.IITR139 == null && wreChildDocument.WRE47.GetValueOrDefault() != 0)));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.310101",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Work related self-education expenses amount is incorrect",
                        LongDescription = @"The 'Work related self-education expenses' amount must equal the sum of the 'Self education expenses - Net deduction' amounts on the attached Work Related Expenses schedule",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.EducationNet.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310101" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR139", Value = report.iitrParentDocument.IITR139.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE47", Value = wreChildDocument.WRE47.GetValueOrDefault().ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310101

                #region VR.ATO.IITR.310102

                /* VR.ATO.IITR.310102
                Work related car expenses amount is present and a Schedule of Work Related Expenses is attached, however, car expenses comments, or car expenses description, or car expenses amount claimed, or car expenses method used code is missing from the schedule.

                Legacy Rule Format:
                IF ([iitr134] > 0) AND (COUNT(SCHEDULE = "WRE") = 1) AND (([wre12] = NULLORBLANK) OR (COUNT(WRE:RP.{CarSeqNum}) = 0) OR ANY OCCURRENCE OF(WRE:RP.{CarSeqNum}) = (WRE:RP.{CarSeqNum} WHERE [wre13] = NULLORBLANK OR [wre22] = NULL OR [wre15] = NULLORBLANK))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                IITR:RP:^IITR134          = tns:IncomeTax.Deduction.Car.Amount
                WRE:RP:^WRE12             = tns:IncomeTax.Deduction.CarReason.Description
                WRE:RP.{CarSeqNum}:^WRE13 = tns:Assets.CarMakeModel.Description
                WRE:RP.{CarSeqNum}:^WRE15 = tns:IncomeTax.Deduction.CarCalculationMethod.Code
                WRE:RP.{CarSeqNum}:^WRE22 = tns:IncomeTax.Deduction.Car.Amount
                */
                assertion = (report.iitrParentDocument.IITR134.GetValueOrDefault() > 0 && (string.IsNullOrEmpty(wreChildDocument.WRE12) || Count(wreChildDocument.RPCarSeqNumContextCollection) == 0 || (Count(wreChildDocument.RPCarSeqNumContextCollection) > 0 && wreChildDocument.RPCarSeqNumContextCollection.Any(context => string.IsNullOrEmpty(context.WRE13) || context.WRE22 == null || string.IsNullOrEmpty(context.WRE15)))));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.310102",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Work related car expense item details on attached Work Related Expenses schedule required",
                        LongDescription = @"When an amount above zero for 'Work related car expenses' is provided, the 'Car expenses - Explanation and justification for work purposes', 'Car expenses - Description of car (make, model)', 'Car expenses - Amount' and 'Car expenses - Claim code' work related car expense item details must be provided on the attached Work Related Expenses schedule",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.Car.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310102" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR134", Value = report.iitrParentDocument.IITR134.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE12", Value = wreChildDocument.WRE12 });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE13", Value = (Count(wreChildDocument.RPCarSeqNumContextCollection) > 0 && wreChildDocument.RPCarSeqNumContextCollection.Any(context => string.IsNullOrEmpty(context.WRE13) || context.WRE22 == null || string.IsNullOrEmpty(context.WRE15)) ? wreChildDocument.RPCarSeqNumContextCollection.Find(context => string.IsNullOrEmpty(context.WRE13) || context.WRE22 == null || string.IsNullOrEmpty(context.WRE15)).WRE13 : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE15", Value = (Count(wreChildDocument.RPCarSeqNumContextCollection) > 0 && wreChildDocument.RPCarSeqNumContextCollection.Any(context => string.IsNullOrEmpty(context.WRE13) || context.WRE22 == null || string.IsNullOrEmpty(context.WRE15)) ? wreChildDocument.RPCarSeqNumContextCollection.Find(context => string.IsNullOrEmpty(context.WRE13) || context.WRE22 == null || string.IsNullOrEmpty(context.WRE15)).WRE15 : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE22", Value = (Count(wreChildDocument.RPCarSeqNumContextCollection) > 0 && wreChildDocument.RPCarSeqNumContextCollection.Any(context => string.IsNullOrEmpty(context.WRE13) || context.WRE22 == null || string.IsNullOrEmpty(context.WRE15)) ? wreChildDocument.RPCarSeqNumContextCollection.Find(context => string.IsNullOrEmpty(context.WRE13) || context.WRE22 == null || string.IsNullOrEmpty(context.WRE15)).WRE22.GetValueOrDefault().ToString() : "null") });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310102

                #region VR.ATO.IITR.310103

                /* VR.ATO.IITR.310103
                Other work related travel expenses is present and a Schedule of Work Related Expenses is attached, however, other travel expenses comments, or other travel expenses type, or other travel expenses amount claimed, or other travel expenses additional information code is missing from the schedule.

                Legacy Rule Format:
                IF ([iitr136] > 0) AND (COUNT(SCHEDULE = "WRE") = 1) AND (([wre24] = NULLORBLANK) OR (COUNT(WRE:RP.{TrvlSeqNum}) = 0) OR ANY OCCURRENCE OF(WRE:RP.{TrvlSeqNum}) = (WRE:RP.{TrvlSeqNum} WHERE [wre25] = NULLORBLANK OR [wre29] = NULL OR [wre27] = NULLORBLANK))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                IITR:RP:^IITR136           = tns:IncomeTax.Deduction.Travel.Amount
                WRE:RP:^WRE24              = tns:IncomeTax.Deduction.TravelReason.Description
                WRE:RP.{TrvlSeqNum}:^WRE25 = tns:IncomeTax.Deduction.TravelCategory.Code
                WRE:RP.{TrvlSeqNum}:^WRE27 = tns:IncomeTax.Deduction.TravelStandardEvidenceCalculation.Code
                WRE:RP.{TrvlSeqNum}:^WRE29 = tns:IncomeTax.Deduction.Travel.Amount
                */
                assertion = (report.iitrParentDocument.IITR136.GetValueOrDefault() > 0 && (string.IsNullOrEmpty(wreChildDocument.WRE24) || Count(wreChildDocument.RPTrvlSeqNumContextCollection) == 0 || (Count(wreChildDocument.RPTrvlSeqNumContextCollection) > 0 && wreChildDocument.RPTrvlSeqNumContextCollection.Any(context => string.IsNullOrEmpty(context.WRE25) || context.WRE29 == null || string.IsNullOrEmpty(context.WRE27)))));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.310103",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Work related other travel expense item details on attached Work Related Expenses schedule required",
                        LongDescription = @"When an amount above zero for 'Work related travel expenses' is provided, the 'Other travel expenses - Explain how these expenses relate to your work', 'Other travel expenses - Category of expense ', 'Other travel expenses - Amount' and 'Other travel expenses - Standard evidence/calculation code' work related other travel expense item details must be provided on the attached Work Related Expenses schedule",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.Travel.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310103" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR136", Value = report.iitrParentDocument.IITR136.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE24", Value = report.wreChildDocuments[0].WRE24 });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE25", Value = (Count(wreChildDocument.RPTrvlSeqNumContextCollection) > 0 && wreChildDocument.RPTrvlSeqNumContextCollection.Any(context => string.IsNullOrEmpty(context.WRE25) || context.WRE29 == null || string.IsNullOrEmpty(context.WRE27)) ? wreChildDocument.RPTrvlSeqNumContextCollection.Find(context => string.IsNullOrEmpty(context.WRE25) || context.WRE29 == null || string.IsNullOrEmpty(context.WRE27)).WRE25 : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE27", Value = (Count(wreChildDocument.RPTrvlSeqNumContextCollection) > 0 && wreChildDocument.RPTrvlSeqNumContextCollection.Any(context => string.IsNullOrEmpty(context.WRE25) || context.WRE29 == null || string.IsNullOrEmpty(context.WRE27)) ? wreChildDocument.RPTrvlSeqNumContextCollection.Find(context => string.IsNullOrEmpty(context.WRE25) || context.WRE29 == null || string.IsNullOrEmpty(context.WRE27)).WRE27 : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE29", Value = (Count(wreChildDocument.RPTrvlSeqNumContextCollection) > 0 && wreChildDocument.RPTrvlSeqNumContextCollection.Any(context => string.IsNullOrEmpty(context.WRE25) || context.WRE29 == null || string.IsNullOrEmpty(context.WRE27)) ? wreChildDocument.RPTrvlSeqNumContextCollection.Find(context => string.IsNullOrEmpty(context.WRE25) || context.WRE29 == null || string.IsNullOrEmpty(context.WRE27)).WRE29.GetValueOrDefault().ToString() : "null") });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310103

                #region VR.ATO.IITR.310104

                /* VR.ATO.IITR.310104
                Work related uniforms/protective clothing/laundry expenses is present and a Schedule of Work Related Expenses is attached, however, uniform/protective clothing/laundry expenses comments, or clothing expenses type, or clothing expenses amount claimed, or clothing expenses additional information code is missing from the schedule.

                Legacy Rule Format:
                IF ([iitr137] > 0) AND (COUNT(SCHEDULE = "WRE") = 1) AND (([wre31] = NULLORBLANK) OR (COUNT(WRE:RP.{ClthngSeqNum}) = 0) OR ANY OCCURRENCE OF(WRE:RP.{ClthngSeqNum}) = (WRE:RP.{ClthngSeqNum} WHERE [wre32] = NULLORBLANK OR [wre36] = NULL OR [wre34] = NULLORBLANK))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                IITR:RP:^IITR137             = tns:IncomeTax.Deduction.Clothing.Amount
                WRE:RP:^WRE31                = tns:IncomeTax.Deduction.ClothingReason.Description
                WRE:RP.{ClthngSeqNum}:^WRE32 = tns:IncomeTax.Deduction.ClothingCategory.Code
                WRE:RP.{ClthngSeqNum}:^WRE34 = tns:IncomeTax.Deduction.ClothingStandardEvidenceCalculation.Code
                WRE:RP.{ClthngSeqNum}:^WRE36 = tns:IncomeTax.Deduction.Clothing.Amount
                */
                assertion = (report.iitrParentDocument.IITR137.GetValueOrDefault() > 0 && (string.IsNullOrEmpty(wreChildDocument.WRE31) || Count(wreChildDocument.RPClthngSeqNumContextCollection) == 0 || (Count(wreChildDocument.RPClthngSeqNumContextCollection) > 0 && wreChildDocument.RPClthngSeqNumContextCollection.Any(context => string.IsNullOrEmpty(context.WRE32) || context.WRE36 == null || string.IsNullOrEmpty(context.WRE34)))));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.310104",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Work related uniform expense item details on attached Work Related Expenses schedule required",
                        LongDescription = @"When an amount above zero for 'Work related uniform, occupation specific or protective clothing, laundry and dry cleaning expenses' is provided, the 'Uniform, protective or occupation specific clothing and laundry expenses - Description of clothing and justification for work purposes', 'Clothing expenses - Category', 'Clothing expenses - Amount' and 'Clothing expenses - Standard evidence/calculation information code' work related self-education expense item details must be provided on the attached Work Related Expenses schedule",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.Clothing.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310104" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR137", Value = report.iitrParentDocument.IITR137.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE31", Value = report.wreChildDocuments[0].WRE31 });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE32", Value = (Count(wreChildDocument.RPClthngSeqNumContextCollection) > 0 && wreChildDocument.RPClthngSeqNumContextCollection.Any(context => string.IsNullOrEmpty(context.WRE32) || context.WRE36 == null || string.IsNullOrEmpty(context.WRE34)) ? wreChildDocument.RPClthngSeqNumContextCollection.Find(context => string.IsNullOrEmpty(context.WRE32) || context.WRE36 == null || string.IsNullOrEmpty(context.WRE34)).WRE32 : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE34", Value = (Count(wreChildDocument.RPClthngSeqNumContextCollection) > 0 && wreChildDocument.RPClthngSeqNumContextCollection.Any(context => string.IsNullOrEmpty(context.WRE32) || context.WRE36 == null || string.IsNullOrEmpty(context.WRE34)) ? wreChildDocument.RPClthngSeqNumContextCollection.Find(context => string.IsNullOrEmpty(context.WRE32) || context.WRE36 == null || string.IsNullOrEmpty(context.WRE34)).WRE34 : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE36", Value = (Count(wreChildDocument.RPClthngSeqNumContextCollection) > 0 && wreChildDocument.RPClthngSeqNumContextCollection.Any(context => string.IsNullOrEmpty(context.WRE32) || context.WRE36 == null || string.IsNullOrEmpty(context.WRE34)) ? wreChildDocument.RPClthngSeqNumContextCollection.Find(context => string.IsNullOrEmpty(context.WRE32) || context.WRE36 == null || string.IsNullOrEmpty(context.WRE34)).WRE36.GetValueOrDefault().ToString() : "null") });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310104

                #region VR.ATO.IITR.310105

                /* VR.ATO.IITR.310105
                Work related self-education expenses is present and a Schedule of Work Related Expenses is attached, however, self-education expenses comments, or self-education expenses - name of course, or self-education expenses - institution, or self-education expenses type, or self-education expenses gross amount claimed, or self-education expenses net of expenses amount, or self-education expenses additional information code is missing.

                Legacy Rule Format:
                IF ([iitr139] > 0) AND (COUNT(SCHEDULE = "WRE") = 1) AND (([wre38] = NULLORBLANK) OR ([wre47] = NULL) OR (COUNT(WRE:RP.{EduSeqNum}) = 0) OR ANY OCCURRENCE OF(WRE:RP.{EduSeqNum}) = (WRE:RP.{EduSeqNum} WHERE ([wre39] = NULLORBLANK OR [wre40] = NULLORBLANK OR [wre41] = NULLORBLANK OR [wre45] = NULL OR [wre43] = NULLORBLANK)))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                IITR:RP:^IITR139          = tns:IncomeTax.Deduction.EducationNet.Amount
                WRE:RP:^WRE38             = tns:IncomeTax.Deduction.EducationReason.Description
                WRE:RP:^WRE47             = tns:IncomeTax.Deduction.EducationNet.Amount
                WRE:RP.{EduSeqNum}:^WRE40 = tns:IncomeTax.Identifiers.InstitutionName.Text
                WRE:RP.{EduSeqNum}:^WRE41 = tns:IncomeTax.Deduction.EducationCategory.Code
                WRE:RP.{EduSeqNum}:^WRE43 = tns:IncomeTax.Deduction.EducationStandardEvidenceCalculation.Code
                WRE:RP.{EduSeqNum}:^WRE45 = tns:IncomeTax.Deduction.Education.Amount
                */
                assertion = (report.iitrParentDocument.IITR139.GetValueOrDefault() > 0 && (string.IsNullOrEmpty(wreChildDocument.WRE38) || wreChildDocument.WRE47 == null || Count(wreChildDocument.RPEduSeqNumContextCollection) == 0 || (Count(wreChildDocument.RPEduSeqNumContextCollection) > 0 && wreChildDocument.RPEduSeqNumContextCollection.Any(context => string.IsNullOrEmpty(context.WRE39) || string.IsNullOrEmpty(context.WRE40) || string.IsNullOrEmpty(context.WRE41) || context.WRE45 == null || string.IsNullOrEmpty(context.WRE43)))));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.310105",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Work related self-education expense item details on attached Work Related Expenses schedule required",
                        LongDescription = @"When an amount above zero for 'Work related self-education expenses' is provided, the 'Self education expenses - Explain how expenses relate to current work activities', 'Self education expenses - Name of course', 'Self education expenses - Institution', 'Self education expenses - Category of expense', 'Self education expenses - Gross amount', 'Self education expenses - Net deduction' and 'Self education expenses - Explanation of other evidence/calculation code used' work related self-education expense item details must be provided on the attached Work Related Expenses schedule",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.EducationNet.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310105" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR139", Value = report.iitrParentDocument.IITR139.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE38", Value = report.wreChildDocuments[0].WRE38 });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE40", Value = (Count(wreChildDocument.RPEduSeqNumContextCollection) > 0 && wreChildDocument.RPEduSeqNumContextCollection.Any(context => string.IsNullOrEmpty(context.WRE39) || string.IsNullOrEmpty(context.WRE40) || string.IsNullOrEmpty(context.WRE41) || context.WRE45 == null || string.IsNullOrEmpty(context.WRE43)) ? wreChildDocument.RPEduSeqNumContextCollection.Find(context => string.IsNullOrEmpty(context.WRE39) || string.IsNullOrEmpty(context.WRE40) || string.IsNullOrEmpty(context.WRE41) || context.WRE45 == null || string.IsNullOrEmpty(context.WRE43)).WRE40 : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE41", Value = (Count(wreChildDocument.RPEduSeqNumContextCollection) > 0 && wreChildDocument.RPEduSeqNumContextCollection.Any(context => string.IsNullOrEmpty(context.WRE39) || string.IsNullOrEmpty(context.WRE40) || string.IsNullOrEmpty(context.WRE41) || context.WRE45 == null || string.IsNullOrEmpty(context.WRE43)) ? wreChildDocument.RPEduSeqNumContextCollection.Find(context => string.IsNullOrEmpty(context.WRE39) || string.IsNullOrEmpty(context.WRE40) || string.IsNullOrEmpty(context.WRE41) || context.WRE45 == null || string.IsNullOrEmpty(context.WRE43)).WRE41 : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE43", Value = (Count(wreChildDocument.RPEduSeqNumContextCollection) > 0 && wreChildDocument.RPEduSeqNumContextCollection.Any(context => string.IsNullOrEmpty(context.WRE39) || string.IsNullOrEmpty(context.WRE40) || string.IsNullOrEmpty(context.WRE41) || context.WRE45 == null || string.IsNullOrEmpty(context.WRE43)) ? wreChildDocument.RPEduSeqNumContextCollection.Find(context => string.IsNullOrEmpty(context.WRE39) || string.IsNullOrEmpty(context.WRE40) || string.IsNullOrEmpty(context.WRE41) || context.WRE45 == null || string.IsNullOrEmpty(context.WRE43)).WRE43 : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE45", Value = (Count(wreChildDocument.RPEduSeqNumContextCollection) > 0 && wreChildDocument.RPEduSeqNumContextCollection.Any(context => string.IsNullOrEmpty(context.WRE39) || string.IsNullOrEmpty(context.WRE40) || string.IsNullOrEmpty(context.WRE41) || context.WRE45 == null || string.IsNullOrEmpty(context.WRE43)) ? wreChildDocument.RPEduSeqNumContextCollection.Find(context => string.IsNullOrEmpty(context.WRE39) || string.IsNullOrEmpty(context.WRE40) || string.IsNullOrEmpty(context.WRE41) || context.WRE45 == null || string.IsNullOrEmpty(context.WRE43)).WRE45.GetValueOrDefault().ToString() : "null") });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE47", Value = report.wreChildDocuments[0].WRE47.GetValueOrDefault().ToString() });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310105
                
                #region VR.ATO.IITR.310132

                /* VR.ATO.IITR.310132
                A Schedule of Work Related Expenses is attached and the car expenses amount claimed is not equal to the amount of work related car expenses in the Individual return.

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "WRE") = 1) AND ((([iitr134] <> NULL) AND ([iitr134] <> SUM([wre22]))) OR (([iitr134] = NULL) AND (SUM([wre22]) <> 0)))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                IITR:RP:^IITR134          = tns:IncomeTax.Deduction.Car.Amount
                WRE:RP.{CarSeqNum}:^WRE22 = tns:IncomeTax.Deduction.Car.Amount
                */
                assertion = ((report.iitrParentDocument.IITR134 != null && (Count(wreChildDocument.RPCarSeqNumContextCollection) > 0 ? report.iitrParentDocument.IITR134.GetValueOrDefault() != wreChildDocument.RPCarSeqNumContextCollection.Sum(context => context.WRE22.GetValueOrDefault()) : report.iitrParentDocument.IITR134.GetValueOrDefault() != 0)) || (report.iitrParentDocument.IITR134 == null && (Count(wreChildDocument.RPCarSeqNumContextCollection) > 0 ? wreChildDocument.RPCarSeqNumContextCollection.Sum(context => context.WRE22.GetValueOrDefault()) != 0 : false)));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.310132",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Work related car expenses amount is incorrect",
                        LongDescription = @"The 'Work related car expenses' amount must equal the sum of the 'Car expenses - Amount' amounts on the attached Work Related Expenses schedule",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.Car.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310132" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR134", Value = report.iitrParentDocument.IITR134.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE22", Value = (Count(wreChildDocument.RPCarSeqNumContextCollection) > 0 ? wreChildDocument.RPCarSeqNumContextCollection.Sum(context => context.WRE22.GetValueOrDefault()).ToString() : "0") });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310132

                #region VR.ATO.IITR.310133

                /* VR.ATO.IITR.310133
                A Schedule of Work Related Expenses is attached and the other travel expenses amount is not equal to the amount of other work related travel expenses in the Individual return.

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "WRE") = 1) AND ((([iitr136] <> NULL) AND ([iitr136] <> SUM([wre29]))) OR (([iitr136] = NULL) AND (SUM([wre29]) <> 0)))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                IITR:RP:^IITR136           = tns:IncomeTax.Deduction.Travel.Amount
                WRE:RP.{TrvlSeqNum}:^WRE28 = tns:IncomeTax.Deduction.Travel.Amount
                */
                assertion = ((report.iitrParentDocument.IITR136 != null && (Count(wreChildDocument.RPTrvlSeqNumContextCollection) > 0 ? report.iitrParentDocument.IITR136.GetValueOrDefault() != wreChildDocument.RPTrvlSeqNumContextCollection.Sum(context => context.WRE29.GetValueOrDefault()) : report.iitrParentDocument.IITR136.GetValueOrDefault() != 0)) || (report.iitrParentDocument.IITR136 == null && (Count(wreChildDocument.RPTrvlSeqNumContextCollection) > 0 ? wreChildDocument.RPTrvlSeqNumContextCollection.Sum(context => context.WRE29.GetValueOrDefault()) != 0 : false)));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.310133",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Work related travel expenses amount is incorrect",
                        LongDescription = @"The 'Work related travel expenses' amount must equal the sum of the 'Other travel expenses - Amount' amounts on the attached Work Related Expenses schedule",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.Travel.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310133" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR136", Value = report.iitrParentDocument.IITR136.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE29", Value = (Count(wreChildDocument.RPTrvlSeqNumContextCollection) > 0 ? wreChildDocument.RPTrvlSeqNumContextCollection.Sum(context => context.WRE29.GetValueOrDefault()).ToString() : "0") });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310133

                #region VR.ATO.IITR.310134

                /* VR.ATO.IITR.310134
                A Schedule of Work Related Expenses is attached and the clothing expenses amount claimed is not equal to the amount of work related uniforms/protective clothing/ laundry expenses in the Individual return.

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "WRE") = 1) AND ((([iitr137] <> NULL) AND ([iitr137] <> SUM([wre36]))) OR (([iitr137] = NULL) AND (SUM([wre36]) <> 0)))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                IITR:RP:^IITR137             = tns:IncomeTax.Deduction.Clothing.Amount
                WRE:RP.{ClthngSeqNum}:^WRE36 = tns:IncomeTax.Deduction.Clothing.Amount
                */
                assertion = ((report.iitrParentDocument.IITR137 != null && (Count(wreChildDocument.RPClthngSeqNumContextCollection) > 0 ? report.iitrParentDocument.IITR137.GetValueOrDefault() != wreChildDocument.RPClthngSeqNumContextCollection.Sum(context => context.WRE36.GetValueOrDefault()) : report.iitrParentDocument.IITR137.GetValueOrDefault() != 0)) || (report.iitrParentDocument.IITR137 == null && (Count(wreChildDocument.RPClthngSeqNumContextCollection) > 0 ? wreChildDocument.RPClthngSeqNumContextCollection.Sum(context => context.WRE36.GetValueOrDefault()) != 0 : false)));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.310134",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"'Work related uniform, occupation specific or protective clothing,... expenses' amount is incorrect",
                        LongDescription = @"The 'Work related uniform, occupation specific or protective clothing, laundry and dry cleaning expenses' amount must equal the sum of the 'Clothing expenses - Amount' amounts on the attached Work Related Expenses schedule",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.Clothing.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310134" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR137", Value = report.iitrParentDocument.IITR137.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE36", Value = (Count(wreChildDocument.RPClthngSeqNumContextCollection) > 0 ? wreChildDocument.RPClthngSeqNumContextCollection.Sum(context => context.WRE36.GetValueOrDefault()).ToString() : "0") });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310134

                #region VR.ATO.IITR.310135

                /* VR.ATO.IITR.310135
                A Schedule of Work Related Expenses is attached and the amount of other work expenses claimed is not equal to the amount of other work related expenses claimed in the Individual return.

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "WRE") = 1) AND ((([iitr141] <> NULL) AND ([iitr141] <> SUM([wre55]))) OR (([iitr141] = NULL) AND (SUM([wre55]) <> 0)))
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:
                IITR:RP:^IITR141            = tns:IncomeTax.Deduction.WorkRelatedOther.Amount
                WRE:RP.{OTHWRSeqNum}:^WRE55 = tns:IncomeTax.Deduction.WorkRelatedOther.Amount
                */
                assertion = ((report.iitrParentDocument.IITR141 != null && (Count(wreChildDocument.RPOTHWRSeqNumContextCollection) > 0 ? report.iitrParentDocument.IITR141.GetValueOrDefault() != wreChildDocument.RPOTHWRSeqNumContextCollection.Sum(context => context.WRE55.GetValueOrDefault()) : report.iitrParentDocument.IITR141.GetValueOrDefault() != 0)) || (report.iitrParentDocument.IITR141 == null && (Count(wreChildDocument.RPOTHWRSeqNumContextCollection) > 0 ? wreChildDocument.RPOTHWRSeqNumContextCollection.Sum(context => context.WRE55.GetValueOrDefault()) != 0 : false)));
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.310135",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Other work related expenses amount is incorrect",
                        LongDescription = @"The 'Other work related expenses' amount must equal the sum of the 'Other work expenses - Amount' amounts on the attached Work Related Expenses schedule",
                        Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.WorkRelatedOther.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310135" } },
                    };
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "IITR141", Value = report.iitrParentDocument.IITR141.GetValueOrDefault().ToString() });
                    processMessage.Parameters.Add(new ProcessMessageParameter
                    { Name = "WRE55", Value = (Count(wreChildDocument.RPOTHWRSeqNumContextCollection) > 0 ? wreChildDocument.RPOTHWRSeqNumContextCollection.Sum(context => context.WRE55.GetValueOrDefault()).ToString() : "0") });

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.310135

                #region VR.ATO.IITR.600043

                /* VR.ATO.IITR.600043
                Invalid context. Period start date for the Work Related Expenses schedule must be the same as the Period start date of the parent form.

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "WRE") = 1) AND (WRE:RP:period.startDate <> RP:period.startDate)
                    RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^StartDate = RP
                WRE:RP:^StartDate  = RP
                */
                assertion = (wreChildDocument.RPStartDate != report.iitrParentDocument.RPStartDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.600043",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Period start date for the Work Related Expenses schedule is invalid",
                        LongDescription = @"Invalid context. Period start date for the Work Related Expenses schedule must be the same as the Period start date of the parent form.",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + report.wreChildDocuments[0].RPId + "']/xbrli:period/xbrli:startDate",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.600043" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.600043

                #region VR.ATO.IITR.600044

                /* VR.ATO.IITR.600044
                Invalid context. Period end date for the Work Related Expenses schedule must be the same as the Period end date of the parent form.

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "WRE") = 1) AND (WRE:RP:period.endDate <> RP:period.endDate)
                    RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^EndDate = RP
                WRE:RP:^EndDate  = RP
                */
                assertion = (wreChildDocument.RPEndDate != report.iitrParentDocument.RPEndDate);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.600044",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Period end date for the Work Related Expenses schedule is invalid",
                        LongDescription = @"Invalid context. Period end date for the Work Related Expenses schedule must be the same as the Period end date of the parent form.",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + report.wreChildDocuments[0].RPId + "']/xbrli:period/xbrli:endDate",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.600044" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.600044

                #region VR.ATO.IITR.600045

                /* VR.ATO.IITR.600045
                The TFN provided for the Work Related Expenses schedule should be the same as the TFN of the parent form.

                Legacy Rule Format:
                IF (COUNT(SCHEDULE = "WRE") = 1) AND (WRE:RP:entity.identifier.TFN <> RP:entity.identifier.TFN)
                   RETURN VALIDATION MESSAGE
                ENDIF

                Data Elements:

                IITR:RP:^EntityTFN = RP
                WRE:RP:^EntityTFN  = RP
                */
                assertion = (wreChildDocument.RPIdentifierTFN != report.iitrParentDocument.RPIdentifierTFN);
                if (assertion)
                {
                    processMessage = new ProcessMessageDocument()
                    {
                        Code = "CMN.ATO.IITR.600045",
                        Severity = ProcessMessageSeverity.Error,
                        Description = @"Tax file number invalid",
                        LongDescription = @"The TFN provided for the Work Related Expenses schedule must be the same as the TFN of the parent form",
                        Location = "/xbrli:xbrl/xbrli:context[@id='" + report.wreChildDocuments[0].RPId + "']/xbrli:entity/xbrli:identifier",
                        Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.600045" } },
                    };

                    response.Add(processMessage);
                }

                #endregion // VR.ATO.IITR.600045

                #region Repeating RPOTHWRSeqNumContext

                if (wreChildDocument.RPOTHWRSeqNumContextCollection != null)
                {
                    foreach (WRE2017.RPOTHWRSeqNumContext wreOTHWRContext in wreChildDocument.RPOTHWRSeqNumContextCollection)
                    {
                        #region VR.ATO.IITR.310106

                        /* VR.ATO.IITR.310106
                        Other work related travel expenses is present and a Schedule of Work Related Expenses is attached and other work expenses type is indicated, however, other work expenses comments, or other work expenses information code is missing from the schedule.

                        Legacy Rule Format:
                        IF ([iitr141] > 0) AND (COUNT(SCHEDULE = "WRE") = 1) AND (COUNT(WRE:RP.{OTHWRSeqNum}) > 0) AND (ANY OCCURRENCE OF([wre50]) <> NULLORBLANK) AND (([wre49] = NULLORBLANK) OR (ANY OCCURRENCE OF(WRE:RP.{OTHWRSeqNum}) = (WRE:RP.{OTHWRSeqNum} WHERE (([wre50] <> NULLORBLANK) AND ([wre55] = NULL OR [wre53] = NULLORBLANK)))))
                           RETURN VALIDATION MESSAGE
                        ENDIF

                        Data Elements:
                        IITR:RP:^IITR141            = tns:IncomeTax.Deduction.WorkRelatedOther.Amount
                        WRE:RP:^WRE49               = tns:IncomeTax.Deduction.WorkRelatedReasonOther.Description
                        WRE:RP.{OTHWRSeqNum}:^WRE50 = tns:IncomeTax.Deduction.WorkRelatedCategoryOther.Code
                        WRE:RP.{OTHWRSeqNum}:^WRE53 = tns:IncomeTax.Deduction.WorkRelatedStandardEvidenceCalculationOther.Code
                        WRE:RP.{OTHWRSeqNum}:^WRE55 = tns:IncomeTax.Deduction.WorkRelatedOther.Amount
                        */
                        assertion = (report.iitrParentDocument.IITR141.GetValueOrDefault() > 0 && Count(wreChildDocument.RPOTHWRSeqNumContextCollection) > 0 && !string.IsNullOrEmpty(wreOTHWRContext.WRE50) && (string.IsNullOrEmpty(wreChildDocument.WRE49) || (!string.IsNullOrEmpty(wreOTHWRContext.WRE50) && (wreOTHWRContext.WRE55 == null || string.IsNullOrEmpty(wreOTHWRContext.WRE53)))));
                        if (assertion)
                        {
                            processMessage = new ProcessMessageDocument()
                            {
                                Code = "CMN.ATO.IITR.310106",
                                Severity = ProcessMessageSeverity.Error,
                                Description = @"Other work related expense item details on attached Work Related Expenses schedule required",
                                LongDescription = @"When an amount above zero for 'Other work related expenses' is provided, as well as 'Other work expenses - Category of expense' on the attached Work Related Expenses schedule, the other work related expense item details, 'Other work expenses - Explanation and justification for work purposes of each expense', 'Other work expenses - Amount' and 'Other work expenses - Standard evidence/calculation information code', must be provided on the attached Work Related Expenses schedule",
                                Location = "/xbrli:xbrl/tns:IncomeTax.Deduction.WorkRelatedOther.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                                Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.310106" } },
                            };
                            processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "IITR141", Value = report.iitrParentDocument.IITR141.GetValueOrDefault().ToString() });
                            processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "WRE49", Value = report.wreChildDocuments[0].WRE49 });
                            processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "WRE50", Value = (wreChildDocument.RPOTHWRSeqNumContextCollection.Any(context => string.IsNullOrEmpty(context.WRE50) && (context.WRE55 == null || string.IsNullOrEmpty(context.WRE53))) ? wreChildDocument.RPOTHWRSeqNumContextCollection.Find(context => string.IsNullOrEmpty(context.WRE50) && (context.WRE55 == null || string.IsNullOrEmpty(context.WRE53))).WRE50 : "null") });
                            processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "WRE53", Value = (wreChildDocument.RPOTHWRSeqNumContextCollection.Any(context => string.IsNullOrEmpty(context.WRE50) && (context.WRE55 == null || string.IsNullOrEmpty(context.WRE53))) ? wreChildDocument.RPOTHWRSeqNumContextCollection.Find(context => string.IsNullOrEmpty(context.WRE50) && (context.WRE55 == null || string.IsNullOrEmpty(context.WRE53))).WRE53 : "null") });
                            processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "WRE55", Value = (wreChildDocument.RPOTHWRSeqNumContextCollection.Any(context => string.IsNullOrEmpty(context.WRE50) && (context.WRE55 == null || string.IsNullOrEmpty(context.WRE53))) ? wreChildDocument.RPOTHWRSeqNumContextCollection.Find(context => string.IsNullOrEmpty(context.WRE50) && (context.WRE55 == null || string.IsNullOrEmpty(context.WRE53))).WRE55.GetValueOrDefault().ToString() : "null") });

                            response.Add(processMessage);
                        }

                        #endregion // VR.ATO.IITR.310106
                    }
                }
                
                #endregion // Repeating RPOTHWRSeqNumContext

                #region Repeating RPTrvlSeqNumContext

                if (wreChildDocument.RPTrvlSeqNumContextCollection != null)
                {
                    foreach (WRE2017.RPTrvlSeqNumContext wreTravelContext in wreChildDocument.RPTrvlSeqNumContextCollection)
                    {
                        #region VR.ATO.IITR.300064

                        /* VR.ATO.IITR.300064
                        A schedule of work related expenses is attached and other travel expenses type is 'Domestic', 'Overseas' and other expenses additional information code is 'Allowance' and allowance, earnings, tips is missing or equal to zero in the Individual return.

                        Legacy Rule Format:
                        WHERE IN (WRE:RP.{TrvlSeqNum})
                        IF (COUNT(SCHEDULE = "WRE") = 1) AND ([wre25] = SET("Domestic","Overseas")) AND ([wre27] = "Allowance") AND (([iitr65] = NULL) OR ([iitr65] = 0))
                           RETURN VALIDATION MESSAGE
                        ENDIF

                        Data Elements:
                        IITR:RP:^IITR65            = tns:Income.AllowancesEarningsTipsDirectorsFees.Amount
                        WRE:RP.{TrvlSeqNum}:^WRE25 = tns:IncomeTax.Deduction.TravelCategory.Code
                        WRE:RP.{TrvlSeqNum}:^WRE27 = tns:IncomeTax.Deduction.TravelStandardEvidenceCalculation.Code
                        */
                        assertion = (IsMatch(wreTravelContext.WRE25, "^(Domestic|Overseas)$") && wreTravelContext.WRE27 == "Allowance" && (report.iitrParentDocument.IITR65 == null || report.iitrParentDocument.IITR65.GetValueOrDefault() == 0));
                        if (assertion)
                        {
                            processMessage = new ProcessMessageDocument()
                            {
                                Code = "CMN.ATO.IITR.300064",
                                Severity = ProcessMessageSeverity.Error,
                                Description = @"Amount of allowances, earnings, tips, director's fees, etc is required",
                                LongDescription = @"When ""Domestic"" or ""Overseas"" is selected for 'Other travel expenses - Category of expense', and ""Allowance"" is selected for 'Other travel expenses - Standard evidence/calculation code', both on the attached Work related expenses schedule, an amount above zero must be provided for 'Amount of allowances, earnings, tips, director's fees, etc'",
                                Location = "/xbrli:xbrl/tns:Income.AllowancesEarningsTipsDirectorsFees.Amount[@contextRef='" + report.iitrParentDocument.RPId + "']",
                                Parameters = new ProcessMessageParameters() { new ProcessMessageParameter() { Name = "RuleIdentifier", Value = "VR.ATO.IITR.300064" } },
                            };
                            processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "IITR65", Value = report.iitrParentDocument.IITR65.GetValueOrDefault().ToString() });
                            processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "WRE25", Value = wreTravelContext.WRE25 });
                            processMessage.Parameters.Add(new ProcessMessageParameter
                            { Name = "WRE27", Value = wreTravelContext.WRE27 });

                            response.Add(processMessage);
                        }

                        #endregion // VR.ATO.IITR.300064
                    }
                }
                
                #endregion // Repeating RPTrvlSeqNumContext
            }

            #endregion // WRE Child Present

            foreach (ProcessMessageDocument currentProcessMessage in response)
            {
                if (currentProcessMessage.Parameters != null)
                {
                    foreach (ProcessMessageParameter currentParameter in currentProcessMessage.Parameters)
                    {
                        if (string.IsNullOrEmpty(currentParameter.Name))
                        {
                            currentParameter.Name = _emptyParameterValue;
                        }

                        if (currentParameter.Name.Length > _maxParameterNameLength)
                        {
                            currentParameter.Name = currentParameter.Name.Substring(0, _maxParameterNameLength - 1);
                        }

                        if (string.IsNullOrEmpty(currentParameter.Value))
                        {
                            currentParameter.Name = _emptyParameterValue;
                        }

                        if (!string.IsNullOrEmpty(currentParameter.Value) && currentParameter.Value.Length > _maxParameterValueLength)
                        {
                            currentParameter.Value = currentParameter.Value.Substring(0, _maxParameterValueLength - 1);
                        }
                    }
                }
            }

            return response;
        }
    }
}